"""
KEGG api interface.

"""
from __future__ import absolute_import

from contextlib import closing

from .service import web_service
from .types import *

class KeggApi(object):
    """ KEGG API """
    
    def __init__(self):
        self.service = web_service()
        
    ##################
    # Meta information
    ##################
    
    def list_databases(self):
        """ Returns a list of available databases.
        
        >>> api.list_databases()
        [Definition(entry_id='nt',...
         
        """
        return map(Definition.from_items, self.service.list_databases())
    
    def list_organisms(self):
        """ Return a list of all available organisms
        
        >>> api.list_organisms()
        [Definition(entry_id='hsa',...
        
        """
        return map(Definition.from_items, self.service.list_organisms())
    
    def list_pathways(self, organism):
        """ Return a list of all available pathways for `organism`
        
        >>> api.list_pathways("hsa")
        [Definition(entry_id=',...
        
        """
        return map(Definition.from_items, self.service.list_pathways(organism))
        
    #######
    # DBGET
    #######
     
    def binfo(self, db):
        """ Return info for database `db`
        
        >>> print api.dbinfo("gb")
        genbank          GenBank nucleic acid sequence database
        gb               Release 186.0, Oct 11
                         National Center for Biotechnology Information
                         144,458,648 entries, 132,067,413,372 bases
                         Last update: 11/10/24
                         <dbget> <fasta> <blast>
                         
        """
        result = self.service.binfo(db)
        if result is not None:
            return BInfo.from_text(str(result))
        else:
            return result
    
    def bfind(self, db, keywords):
        """ Search database 'db' for keywords
        """
        result = self.service.bfind(" ".join([db, keywords]))
        if result is not None:
            return str(result)
        else:
            return result
    
    def bget(self, ids):
        """
        """
        if not isinstance(ids, basestring):
            # Sequence of ids
            ids = " ".join(ids)
        result = self.service.bget(ids)
        if result is not None:
            return str(result)
        else:
            return result
    
    def btit(self, ids):
        """
        """
        if not isinstance(ids, basestring):
            ids = " ".join(ids)
            
        result = self.service.btit(ids)
        if result is not None:
            return str(result)
        else:
            return result
    
    def bconv(self, ids):
        if not isinstance(ids, basestring):
            ids = " ".join(ids)
            
        result = self.service.bconv(ids)
        if result is not None:
            return str(result)
        else:
            return result
    
    ########
    # LinkDB
    ########
    
    def get_linkdb_by_entry(self, entry_id, db, offset, limit):
        links = self.service.get_linkdb_by_entry(entry_id, db, offset, limit)
        return [LinkDBRelation(**d) for d in \
                map(dict, links)]
        
    def get_linkdb_between_databases(self, from_db, to_db, offset, limit):
        links = self.service.get_linkdb_between_databases(from_db, to_db, offset, limit)
        return [LinkDBRelation(**d) for d in \
                map(dict, links)]
        
    def get_genes_by_enzyme(self, enzyme_id, org):
        return self.service.get_genes_by_enzyme(enzyme_id, org)
    
    def get_enzymes_by_gene(self, genes_id):
        return self.service.get_enzymes_by_gene(genes_id)
    
    def get_enzymes_by_compound(self, compound_id):
        return self.service.get_enzymes_by_compound(compound_id)
    
    def get_enzymes_by_glycan(self, glycan_id):
        return self.service.get_enzymes_by_glycan(glycan_id)
    
    def get_enzymes_by_reaction(self, reaction_id):
        return self.service.get_enzymes_by_reaction(reaction_id)
    
    def get_compounds_by_enzyme(self, enzyme_id):
        return self.service.get_compounds_by_enzyme(enzyme_id)
    
    def get_compounds_by_reaction(self, reaction_id):
        return self.service.get_compounds_by_reaction(reaction_id)
    
    def get_glycans_by_enzyme(self, enzyme_id):
        return self.service.get_glycans_by_enzyme(enzyme_id)
    
    def get_glycans_by_reaction(self, reaction_id):
        return self.service.get_glycans_by_reaction(reaction_id)
    
    def get_reactions_by_enzyme(self, enzyme_id):
        return self.service.get_reactions_by_enzyme(enzyme_id)
    
    def get_reactions_by_compound(self, compound_id):
        return self.service.get_reactions_by_compound(compound_id)
    
    def get_reactions_by_glycan(self, glycan_id):
        return self.service.get_reactions_by_glycan(glycan_id)
    
    ######
    # SSDB
    ######
    
    def get_best_best_neighbors_by_gene(self, genes_id, offset, limit):
        ssr = self.service.get_best_best_neighbors_by_gene(genes_id, offset, limit)
        return [SSDBRelation(**d) for d in \
                map(dict, ssr)]
    
    def get_best_neighbors_by_gene(self, genes_id, offset, limit):
        ssr = self.service.get_best_neighbors_by_gene(genes_id, offset, limit)
        return [SSDBRelation(**d) for d in \
                map(dict, ssr)]
    
    def get_reverse_best_neighbors_by_gene(self, genes_id, offset, limit):
        ssr = self.service.get_reverse_best_neighbors_by_gene(genes_id, offset, limit)
        return [SSDBRelation(**d) for d in \
                map(dict, ssr)]
    
    def get_paralogs_by_gene(self, genes_id, offset, limit):
        ssr =  self.service.get_paralogs_by_gene(genes_id, offset, limit)
        return [SSDBRelation(**d) for d in \
                map(dict, ssr)]
    
    #######
    # Motif
    #######
    
    def get_motifs_by_gene(self, genes_id, db):
        motif = self.service.get_motifs_by_gene(genes_id, db)
        return [MotifResult(**d) for d in \
                map(dict, motif)]
    
    def get_genes_by_motifs(self, motif_id_list, offset, limit):
        genes = self.service.get_genes_by_motifs(motif_id_list, offset, limit)
        return [Definition(**d) for d in \
                map(dict, genes)]
    
    ####
    # KO
    ####
    
    def get_ko_by_gene(self, genes_id):
        return self.service.get_ko_by_gene(genes_id)
    
    def get_ko_by_ko_class(self, ko_class_id):
        return self.service.get_ko_by_ko_class(ko_class_id)
    
    def get_genes_by_ko_class(self, ko_class_id, org, offset, limit):
        return self.service.get_genes_by_ko_class(ko_class_id, org, offset, limit)
    
    def get_genes_by_ko(self, ko_id, org):
        return self.service.get_genes_by_ko(ko_id, org)
    
    #########
    # Pathway
    #########
    
    def mark_pathway_by_objects(self, pathway_id, object_id_list):
        return self.service.mark_pathway_by_objects(pathway_id, object_id_list)
    
    def color_pathway_by_objects(self, pathway_id, object_id_list, fg_color_list, bg_color_list):
        return self.service.color_pathway_by_objects(pathway_id, object_id_list, fg_color_list, bg_color_list)
    
    def color_pathway_by_elements(self, pathway_id, element_id_list, fg_color_list, bg_color_list):
        return self.service.color_pathway_by_elements(pathway_id, element_id_list, fg_color_list, bg_color_list)
    
    def get_html_of_marked_pathway_by_objects(self, pathway_id, object_id_list):
        return self.service.get_html_of_marked_pathway_by_objects(pathway_id, object_id_list)
    
    def get_html_of_colored_pathway_by_objects(self, pathway_id, object_id_list, fg_color_list, bg_color_list):
        return self.service.get_html_of_colored_pathway_by_objects(pathway_id, object_id_list, fg_color_list, bg_color_list)
    
    def get_html_of_colored_pathway_by_elements(self, pathway_id, element_id_list, fg_color_list, bg_color_list):
        return self.service.get_html_of_colored_pathway_by_elements(pathway_id, element_id_list, fg_color_list, bg_color_list)
    
    def get_references_by_pathway(self, pathway_id):
        return self.service.get_references_by_pathway(pathway_id)
    
    def get_element_relations_by_pathway(self, pathway_id):
        return self.service.get_element_relations_by_pathway(pathway_id)
    
    
    
    def get_genes_by_organism(self, organism, offset=None, limit=None):
        if offset is None and limit is None:
            offset = 0
            limit = self.get_number_of_genes_by_organism(organism)
            
        return self.service.get_genes_by_organism(organism, offset, limit)
    
    def get_number_of_genes_by_organism(self, organism):
        return self.service.get_number_of_genes_by_organism(organism)
    
    ####################
    # Objects by pathway
    ####################
    
    def get_elements_by_pathway(self, pathway_id):
        return self.service.get_elements_by_pathway(pathway_id)
    
    def get_genes_by_pathway(self, pathway_id):
        return self.service.get_genes_by_pathway(pathway_id)
    
    def get_enzymes_by_pathway(self, pathway_id):
        return self.service.get_enzymes_by_pathway(pathway_id)
    
    def get_compounds_by_pathway(self, pathway_id):
        return self.service.get_compounds_by_pathway(pathway_id)
    
    def get_drugs_by_pathway(self, pathway_id):
        return self.service.get_drugs_by_pathway(pathway_id)
    
    def get_glycans_by_pathway(self, pathway_id):
        return self.service.get_glycans_by_pathway(pathway_id)
    
    def get_reactions_by_pathway(self, pathway_id):
        return self.get_reactions_by_pathway(pathway_id)
    
    def get_kos_by_pathway(self, pathway_id):
        return self.service.get_kos_by_pathway(pathway_id)
    
    #####################
    # Pathways by objects
    #####################
    
    def get_pathways_by_genes(self, gene_list):
        return map(str, self.service.get_pathways_by_genes(gene_list))
    
    def get_pathways_by_enzymes(self, enzyme_list):
        return map(str, self.service.get_pathways_by_enzymes(enzyme_list))
    
    def get_pathways_by_compounds(self, compound_list):
        return map(str, self.service.get_pathways_by_compounds(compound_list))
    
    def get_pathways_by_drugs(self, drug_list):
        return map(str, self.service.get_pathways_by_drugs(drug_list))
    
    def get_pathways_by_glycans(self, glycan_list):
        return map(str, self.service.get_pathways_by_glycans(glycan_list))
    
    def get_pathways_by_reactions(self, reaction_list):
        return map(str, self.service.get_pathways_by_reactions(reaction_list))
    
    def get_pathways_by_kos(self, ko_list):
        return map(str, self.service.get_pathways_by_kos(ko_list))
    
    ##########################
    # Relations among pathways
    ##########################
    
    def get_linked_pathways(self, pathway_id):
        if not pathway_id.startswith("path:"):
            pathway_id = "path:" + pathway_id
        return map(str, self.service.get_linked_pathways(pathway_id))
    
    
"""
KEGG api with caching
"""

import os

from . import caching
from .caching import cached_method, cache_entry, touch_dir

try:
    from functools import lru_cache
except ImportError:
    # TODO: move a copy of lru_cache in .caching if distributing this as a
    # standalone package
    from Orange.utils import lru_cache

    
class CachedKeggApi(KeggApi):
    def __init__(self, store=None):
        KeggApi.__init__(self)
        if store is None:
            self.store = {}
    
    # Needed API for cached decorator.
    def cache_store(self):
        from . import conf
        path = conf.params["cache.path"]
        touch_dir(path)
        return caching.Sqlite3Store(os.path.join(path,
                                                 "kegg_api_cache.sqlite3"))
    
    def last_modified(self, args, kwargs=None):
        return getattr(self, "default_release", "")
    
    def set_default_release(self, release):
        self.default_release = release
        
    
    ##################
    # Meta information
    ##################
    
    @lru_cache() # not persistently cached
    def list_databases(self):
        return KeggApi.list_databases(self)
    
    @cached_method
    def list_organisms(self):
        return KeggApi.list_organisms(self)
    
    @cached_method
    def list_pathways(self, organism):
        return KeggApi.list_pathways(self, organism)
    
    #######
    # DBGET
    #######
    
    @lru_cache() # not persistently cached
    def binfo(self, db):
        return KeggApi.binfo(self, db)
    
    @cached_method
    def bfind(self, db, keywords):
        return KeggApi.bfind(self, db, keywords)
    
    @cached_method
    def bget(self, ids):
        rval = KeggApi.bget(self, ids)
        return rval
    
    @cached_method
    def bget(self, ids):
        if not isinstance(ids, basestring):
            return self._batch_bget(ids)
        else:
            return KeggApi.bget(self, ids)
        
    def _batch_bget(self, ids):
        if len(ids) > 100:
            raise ValueError("Can batch at most 100 ids at a time.")
        
        bget = self.bget
        uncached = []
        with closing(bget.cache_store()) as store:
            # Which ids are already cached
            # TODO: Invalidate entries by release string.
            for id in ids:
                key = bget.key_from_args((id,))
                if key not in store:
                    uncached.append(id)
                
        if uncached:
            # in case there are duplicate ids
            uncached = sorted(set(uncached))
            rval = KeggApi.bget(self, uncached)
            if rval is not None:
                entrys = rval.split("///\n")
            else:
                entrys = []
                
            if entrys and not entrys[-1].strip():
                # Delete the last newline if present
                del entrys[-1]
            
            if len(entrys) == len(uncached):
                with closing(bget.cache_store()) as store:
                    for id, entry in zip(uncached, entrys):
                        key = bget.key_from_args((id,))
                        if entry is not None:
                            entry = entry + "///\n"
                        store[key] = cache_entry(entry, mtime=datetime.now())
                        
            else:
                # Try to bisect the uncached list
                if len(uncached) > 1 and len(uncached) - len(entrys) < 4:
                    split = len(uncached) / 2
                    self._batch_bget(uncached[:split])
                    self._batch_bget(uncached[split:])
                else:
                    import warnings
                    warnings.warn("Batch contains invalid ids", UserWarning)
        
        # Finally join all the results, but drop all None objects
        entries = filter(lambda e: e is not None, map(bget, ids))
        
        rval = "".join(entries)
        return rval
    
    @cached_method
    def btit(self, ids):
        return KeggApi.btit(self, ids)
    
    @cached_method
    def bconv(self, ids):
        return KeggApi.bconv(self, ids)
    
    ########
    # LinkDB
    ########
    
    @cached_method
    def get_linkdb_by_entry(self, entry_id, db, offset, limit):
       return KeggApi.get_linkdb_by_entry(self, entry_id, db, offset, limit)
        
    @cached_method
    def get_linkdb_between_databases(self, from_db, to_db, offset, limit):
        return KeggApi.get_linkdb_between_databases(self, from_db, to_db, offset, limit)
            
    @cached_method
    def get_genes_by_enzyme(self, enzyme_id, org):
        return KeggApi.get_genes_by_enzyme(self, enzyme_id, org)
    
    @cached_method
    def get_enzymes_by_gene(self, genes_id):
        return KeggApi.get_enzymes_by_gene(self, genes_id)
    
    @cached_method
    def get_enzymes_by_compound(self, compound_id):
        return KeggApi.get_enzymes_by_compound(self, compound_id)
    
    @cached_method
    def get_enzymes_by_glycan(self, glycan_id):
        return KeggApi.get_enzymes_by_glycan(self, glycan_id)
    
    @cached_method
    def get_enzymes_by_reaction(self, reaction_id):
        return KeggApi.get_enzymes_by_reaction(self, reaction_id)
    
    @cached_method
    def get_compounds_by_enzyme(self, enzyme_id):
        return KeggApi.get_compounds_by_enzyme(self, enzyme_id)
    
    @cached_method
    def get_compounds_by_reaction(self, reaction_id):
        return KeggApi.get_compounds_by_reaction(self, reaction_id)
    
    @cached_method
    def get_glycans_by_enzyme(self, enzyme_id):
        return KeggApi.get_glycans_by_enzyme(self, enzyme_id)
    
    @cached_method
    def get_glycans_by_reaction(self, reaction_id):
        return KeggApi.get_glycans_by_reaction(self, reaction_id)
    
    @cached_method
    def get_reactions_by_enzyme(self, enzyme_id):
        return KeggApi.get_reactions_by_enzyme(self, enzyme_id)
    
    @cached_method
    def get_reactions_by_compound(self, compound_id):
        return KeggApi.get_reactions_by_compound(self, compound_id)
    
    @cached_method
    def get_reactions_by_glycan(self, glycan_id):
        return KeggApi.get_reactions_by_glycan(self, glycan_id)
    
    ######
    # SSDB
    ######
    
    @cached_method
    def get_best_best_neighbors_by_gene(self, genes_id, offset, limit):
        return KeggApi.get_best_best_neighbors_by_gene(self, genes_id, offset, limit)
    
    @cached_method
    def get_best_neighbors_by_gene(self, genes_id, offset, limit):
        return KeggApi.get_best_neighbors_by_gene(self, genes_id, offset, limit)
    
    @cached_method
    def get_reverse_best_neighbors_by_gene(self, genes_id, offset, limit):
        return KeggApi.get_reverse_best_neighbors_by_gene(self, genes_id, offset, limit)
    
    @cached_method
    def get_paralogs_by_gene(self, genes_id, offset, limit):
        return KeggApi.get_paralogs_by_gene(self, genes_id, offset, limit)
    
    #######
    # Motif
    #######
    
    @cached_method
    def get_motifs_by_gene(self, genes_id, db):
        return KeggApi.get_motifs_by_gene(self, genes_id, db)
    
    @cached_method
    def get_genes_by_motifs(self, motif_id_list, offset, limit):
        return KeggApi.get_genes_by_motifs(self, motif_id_list, offset, limit)

    ####
    # KO
    ####
    
    @cached_method
    def get_ko_by_gene(self, genes_id):
        return KeggApi.get_ko_by_gene(self, genes_id)
    
    @cached_method
    def get_ko_by_ko_class(self, ko_class_id):
        return KeggApi.service.get_ko_by_ko_class(self, ko_class_id)
    
    @cached_method
    def get_genes_by_ko_class(self, ko_class_id, org, offset, limit):
        return KeggApi.get_genes_by_ko_class(self, ko_class_id, org, offset, limit)
    
    @cached_method
    def get_genes_by_ko(self, ko_id, org):
        return KeggApi.get_genes_by_ko(self, ko_id, org)
    
    #########
    # Pathway
    #########
    
    # TODO
    
    
    
    @cached_method
    def get_genes_by_organism(self, organism, offset=None, limit=None):
        return KeggApi.get_genes_by_organism(self, organism, offset=offset, limit=limit)
    
    @cached_method
    def get_number_of_genes_by_organism(self, organism):
        return KeggApi.get_number_of_genes_by_organism(self, organism)
     
    @cached_method
    def get_pathways_by_genes(self, gene_list):
        return KeggApi.get_pathways_by_genes(self, gene_list)
    
    @cached_method
    def get_pathways_by_enzymes(self, enzyme_list):
        return KeggApi.get_pathways_by_enzymes(self, enzyme_list)
    
    @cached_method
    def get_pathways_by_compounds(self, compound_list):
        return KeggApi.get_pathways_by_compounds(self, compound_list)
    
    @cached_method
    def get_pathways_by_drugs(self, drug_list):
        return KeggApi.get_pathways_by_drugs(self, drug_list)
    
    @cached_method
    def get_pathways_by_glycans(self, glycan_list):
        return KeggApi.get_pathways_by_glycans(self, glycan_list)
    
    @cached_method
    def get_pathways_by_reactions(self, reaction_list):
        return KeggApi.get_pathways_by_reactions(self, reaction_list)
    
    @cached_method
    def get_pathways_by_kos(self, ko_list):
        return KeggApi.get_pathways_by_kos(self, ko_list)
    
    @cached_method
    def get_elements_by_pathway(self, pathway_id):
        return KeggApi.get_elements_by_pathway(self, pathway_id)
    
    @cached_method
    def get_genes_by_pathway(self, pathway_id):
        return KeggApi.get_genes_by_pathway(self, pathway_id)
    
    @cached_method
    def get_enzymes_by_pathway(self, pathway_id):
        return KeggApi.get_enzymes_by_pathway(self, pathway_id)
    
    @cached_method
    def get_compounds_by_pathway(self, pathway_id):
        return KeggApi.get_compounds_by_pathway(self, pathway_id)
    
    @cached_method
    def get_drugs_by_pathway(self, pathway_id):
        return KeggApi.get_drugs_by_pathway(self, pathway_id)
    
    @cached_method
    def get_glycans_by_pathway(self, pathway_id):
        return KeggApi.get_glycans_by_pathway(self, pathway_id)
    
    @cached_method
    def get_reactions_by_pathway(self, pathway_id):
        return KeggApi.get_reactions_by_pathway(self, pathway_id)
    
    @cached_method
    def get_kos_by_pathway(self, pathway_id):
        return KeggApi.get_kos_by_pathway(self, pathway_id)
    



"""
<name>Gene Atlas Tissue Expression</name>
<description></description>
<prototype>1</prototype>
"""

from __future__ import absolute_import

import os, sys
from collections import defaultdict
import shelve

import Orange
from Orange.utils import lru_cache
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from Orange.orng import orngDataCaching, orngServerFiles

from ... import obiArrayExpress, obiGene, obiGeneAtlas

TAXID_TO_ORG = obiGeneAtlas.TAXID_TO_ORG

class OWGeneAtlasTissueExpression(OWWidget):
    contextHandlers = {"": DomainContextHandler("", ["selected_organism",
                                                     "selected_gene_attr",
                                                     "genes_in_columns",
                                                     "selected_ef",
                                                     "selected_ef_value"])}
    settingsList = []
    
    ORGANISMS = obiArrayExpress.ATLAS_ORGANISMS
    FACTORS = ["Organism part", "Disease state", "Cell type"]
    
    def __init__(self, parent=None, signalManager=None, title="Gene Atlas Tissue Expression"):
        OWWidget.__init__(self, parent, signalManager, title)
        
        self.inputs = [("Example Table", Orange.data.Table, self.set_data)]
        self.outputs = [("Selected Genes", Orange.data.Table)]
        
        self.selected_organism = "Homo sapiens"
        self.selected_gene_attr = 0
        self.genes_in_columns = False
        self.selected_ef = 0
        self.selected_ef_value = 0
        
        self.loadSettings()
        
        #####
        # GUI
        #####
        box = OWGUI.widgetBox(self.controlArea, "Info", addSpace=True)
        self.info_label = OWGUI.widgetLabel(box, "No data on input.\n")
        
        box = OWGUI.widgetBox(self.controlArea, "Organism", addSpace=True)
        cb = OWGUI.comboBox(box, self, "selected_organism",
                            items=self.ORGANISMS,
                            tooltip="Organism name",
                            callback=self.on_organism_change,
                            sendSelectedValue=True,
                            valueType=str
                            )
        cb.setMaximumWidth(250)
        
        box = OWGUI.widgetBox(self.controlArea, "Gene Attribute", addSpace=True)
        self.gene_attr_cb = OWGUI.comboBox(box, self, "selected_gene_attr",
                              tooltip="Attribute (column) containing the gene names.",
                              callback=self.on_gene_attr_change,
                              )
        self.gene_attr_cb.setMaximumWidth(250)
        
        cb = OWGUI.checkBox(box, self, "genes_in_columns", "Use attribute names",
                       tooltip="Gene names in columns.",
                       callback=self.on_genes_change,)
        cb.disables.append((-1, self.gene_attr_cb))
        cb.makeConsistent()
        
        box = OWGUI.widgetBox(self.controlArea, "Tissues", addSpace=True)
        self.categories_cb = OWGUI.comboBox(box, self, "selected_ef",
                                box="Categories", 
                                items=self.FACTORS,
                                tooltip="Experimental factor.",
                                callback=self.on_ef_change,
                                )
        self.categories_cb.box.setFlat(True)
        
        self.values_cb = OWGUI.comboBox(box, self, "selected_ef_value",
                                box="Values",
                                tooltip="Experimental factor value.",
                                callback=self.on_ef_value_change
                                )
        self.values_cb.setMaximumWidth(250)
        self.values_cb.box.setFlat(True)
        
        box = OWGUI.widgetBox(self.controlArea, "Cache", addSpace=True)
        OWGUI.button(box, self, "Clear cache",
                     callback=self.on_cache_clear,
                     tooltip="Clear Gene Atlas cache.")
        
        OWGUI.rubber(self.controlArea)
        
        OWGUI.button(self.controlArea, self, label="Commit",
                     callback=self.commit,
                     tooltip="Send selected genes")
        
        self.report_view = QTreeView(self.mainArea)
        self.report_view.setSelectionMode(QTreeView.ExtendedSelection)
        self.report_view.setSortingEnabled(True)
        self.report_view.setRootIsDecorated(False)
        self.report_view.setAlternatingRowColors(True)
        self.report_view.setEditTriggers(QTreeView.NoEditTriggers)
        self.mainArea.layout().addWidget(self.report_view)
        self.report_header = ["Gene symbol", "Up", "Down"]
        
        model = QStandardItemModel()
        model.setHorizontalHeaderLabels(self.report_header)
        self.report_view.setModel(model)
        
        self.data = None
        self.candidate_vars = []
        self.candidate_var_names = []
        self.results = {}, {}, {}
        
        self.ensembl_info = None
        self.gene_matcher = obiGene.GMDirect()
        self.loaded_matcher_taxid = None
        self.unknown_genes = []
        self.query_genes = []
        
#        self.set_organism(self.selected_organism, update_results=False)
        
        self.get_atlas_summary = obiGeneAtlas.get_atlas_summary
        
        #Cached construct_matcher
        @lru_cache(maxsize=3)
        def my_cached_matcher(org):
            return obiGeneAtlas.default_gene_matcher(org)
        self.construct_matcher = my_cached_matcher
        
    def set_data(self, data=None):
        """ Set the input example table with gene names. 
        """
        self.closeContext("")
        self.clear()
        self.data = data
        if data is not None:
            self.init_gene_box(data)
            taxid = orngDataCaching.data_hints.get_hint(data, key="taxid", default=None)
            if taxid is not None:
                if taxid in TAXID_TO_ORG:
                    self.selected_organism = TAXID_TO_ORG[taxid]
                    
            genesinrows = orngDataCaching.data_hints.get_hint(data, key="genesinrows", default=None)
            if genesinrows is not None:
                self.genes_in_columns = genesinrows
                
            self.openContext("", data)
            self.update_gene_matcher()
            self.update_info_box()
            self.run_query()
            self.update_ef_values_box()
            self.update_report_view()
        else:
            pass
            
    def clear(self):
        """ Clear the widget state.
        """
        self.data = None
        self.gene_attr_cb.clear()
        self.values_cb.clear()
        self.selected_gene_attr = 0
        self.selected_ef_value = 0
        self.candidate_vars = []
        self.candidate_var_names = []
        self.results = {}, {}, {}
        model = QStandardItemModel()
        model.setHorizontalHeaderLabels(self.report_header)
        self.report_view.setModel(model)

    def init_gene_box(self, data):
        """ Populate the 'Gene Attribute' box
        """
        vars = data.domain.variables + data.domain.get_metas().values()
        vars = [var for var in vars if isinstance(var, (Orange.feature.String))]
        self.candidate_vars = vars
        self.gene_attr_cb.addItems([var.name for var in vars])
        # See if any var name contains 'gene'
        gene_in_name = [v for v in vars if "gene" in v.name.lower()]
        if gene_in_name:
            self.selected_gene_attr = vars.index(gene_in_name[-1])
        self.candidate_var_names = [a.name for a in data.domain.attributes]
        if not self.candidate_vars:
            self.genes_in_columns = True            
            
    def on_organism_change(self):
        """ Called when the user changes the organism.
        """
        self.update_gene_matcher()
        self.run_query()
        self.update_ef_values_box()
        self.update_report_view()
    
    def on_gene_attr_change(self):
        """ Called  when the user changes source gene attribute.
        """
        self.update_info_box()
        self.run_query()
        self.update_ef_values_box()
        self.update_report_view()
    
    def on_genes_change(self):
        """ Called when the user toogles the "Genes in columns' check box. 
        """
        self.update_info_box()
        self.run_query()
        self.update_ef_values_box()
        self.update_report_view()
    
    def on_ef_change(self):
        """ Called when the user changes the EF factor.
        """
        self.update_ef_values_box()
        # TODO: restore previous selected value for factor (if value in ef_values)
        self.update_report_view()
        
    def on_ef_value_change(self):
        """ Called when the user changes the EF value.
        """
        results = self.results[self.selected_ef]
        ef_value = self.ef_values[self.selected_ef_value]
        self.update_report_view()
        
    def on_cache_clear(self):
        from contextlib import closing
        with closing(obiGeneAtlas._cache()) as cache:
            cache.clear()
        
    def input_genes(self, full_res=False):
        """ Extract input gene names from ``self.data``. 
        """
        if self.genes_in_columns:
            names = self.candidate_var_names
        else:
            names = []
            attr = self.candidate_vars[self.selected_gene_attr]
            for ex in self.data:
                if not ex[attr].is_special():
                    names.append(str(ex[attr]))
            
        return sorted(set(names))
    
    def match_genes(self, genes, full_res=True):
        """ Match the genes to ensembl gene ids.
        """
        if self.gene_matcher and self.ensembl_info:
            self.gene_matcher.set_targets(self.ensembl_info.keys())
            match_genes = map(self.gene_matcher.match, genes)
            genes = map(self.gene_matcher.umatch, genes)
        else:
            match_genes = None
        genes = sorted(set(genes) - set([None]))
        
        if full_res:
            return genes, match_genes
        else:
            return genes
    
    def run_query(self):
        """ Query the Gene Atlas.
        """
        if self.data is not None:
            genes = self.input_genes()
            matched, match_res = self.match_genes(genes, True)
            all_matched, unknown = [], []
            for gene, match in zip(genes, match_res):
                if len(match) == 1:
                    all_matched.append(match[0])
                elif len(match) > 1:
                    all_matched.append(match[0])
                else:
                    unknown.append(gene)
            self.unknown_genes = unknown
            self.warning(2)
            self.information(2)
            if float(len(unknown)) / len(genes) > 0.5:
                self.warning(2, "%i unknown genes" % len(unknown))
            elif unknown:
                self.information(2, "%i unknown genes" % len(unknown))
                
            gm = obiGene.GMDirect()
            gm.set_targets(all_matched)
            
            # Non threaded
#            self.results = self.get_atlas_summary(all_matched, self.selected_organism, gm)
            # Threaded
            self.error(0)
            self.update_info_box(query_running=True)
            self.progressBarInit()
            self.controlArea.setEnabled(False)
            self.results = None
            try:
                call = self.asyncCall(self.get_atlas_summary, (all_matched,
                                                               self.selected_organism,
                                                               gm),
                                      name="Query Gene Expression Atlas",
                                      onError=self.handle_assync_error)
                QObject.connect(call, SIGNAL("progressChanged(float)"), self.progressBarSet)
                call(progress_callback=call.emitProgressChanged)
                self.results = call.get_result(processEvents=True)
            except obiArrayExpress.GeneAtlasError, ex:
                self.error(0, str(ex))
            finally:
                self.controlArea.setEnabled(True)
                self.update_info_box(query_running=False)
                self.progressBarFinished()
            
            self.query_genes = genes
        
    def update_gene_matcher(self):
        taxid = dict((v,k) for k,v in TAXID_TO_ORG.items()).get(self.selected_organism)
        self.warning(0)
        if taxid != self.loaded_matcher_taxid:
            if taxid:
                self.ensembl_info = obiGene.EnsembleGeneInfo(taxid)
                self.gene_matcher = self.construct_matcher(taxid)
            else:
                self.warning(0, "Cannot match gene names!")
                self.ensembl_info = None
                self.gene_matcher = obiGene.GMDirect()
            self.loded_matcher_taxid = taxid
        
    def update_ef_values_box(self):
        """ Update the "Values" box.
        """
        ef_values = set() 
        results = self.results[self.selected_ef]
        efv_count = defaultdict(int)
        if self.results:
            for gene, val in  results.iteritems():
                for ef_val, (up, down) in val.iteritems():
                   ef_values.add(ef_val)
                   efv_count[ef_val] += up + down
        
        # Sort by up and down count sum of all genes
        self.ef_values = sorted(ef_values, key=efv_count.get, reverse=True)
        self.values_cb.clear()
        self.values_cb.addItems(self.ef_values)
        if self.ef_values:
            self.selected_ef_value = min(len(self.ef_values) - 1, self.selected_ef_value)
        
    def update_report_view(self):
        results = self.results[self.selected_ef]
        
        model = QStandardItemModel()
        
                
        def standard_item(val):
            item = StandardPyItem()
            item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
            item.setData(QVariant(val), Qt.DisplayRole)
            return item
        
        if results and self.ef_values:
            ef_value = self.ef_values[self.selected_ef_value]
            for gene, efvs in results.iteritems():
                if ef_value in efvs:
                    up, down = efvs[ef_value]
                    model.appendRow([standard_item(gene),
                                     standard_item(up),
                                     standard_item(down)])
        
        model.setHorizontalHeaderLabels(self.report_header)
        self.report_view.setModel(model)
        self.report_view.resizeColumnToContents(0)
        self.report_view.resizeColumnToContents(1)
        self.report_view.resizeColumnToContents(2)
        
    def update_info_box(self, query_running=False):
        text = ""
        if self.data is not None:
            genes = self.input_genes()
#            unique, matched = self.match_genes(genes, full_res=True)
            # TODO: How many genes are matched (should update after the query is run).
            if self.results and not query_running:
#                matched = min(max([len(r) for r in self.results]), len(genes))
                matched = len(genes) - len(self.unknown_genes)
                with_results = len(reduce(set.union, [d.keys() for d in self.results], set()))
                text = "{0} gene names on input.\n{1} genes matched.\n{2} genes annotated.".format(len(genes), matched, with_results)
            else:
                text = "{0} gene names on input.\n\n".format(len(genes))
        else:
            text = "No data on input.\n\n"
        if query_running:
            text += "Querying Gene Atlas. Please wait."
        self.info_label.setText(text)
            
    def selected_report_genes(self):
        """ Return the gene names selected in the report view.
        """
        rows = self.report_view.selectionModel().selectedRows(0)
        genes = []
        for index in rows:
            name = str(index.data(Qt.DisplayRole).toString())
            genes.append(name)
        return genes
            
    def commit(self):
        genes = set(self.selected_report_genes())
        self.gene_matcher.set_targets(genes)
        
        genes = [gene.lower() for gene in genes]
        if self.genes_in_columns:
            attrs = [a for a in self.data.domain.attributes \
                     if a.name.lower() in genes or self.gene_matcher.umatch(a.name)]
            domain = Orange.data.Domain(attrs, self.data.domain.class_var)
            domain.add_metas(self.data.domain.get_metas())
            data = Orange.data.Table(domain, self.data)
        else:
            attr = self.candidate_vars[self.selected_gene_attr]
            examples = []
            for ex in self.data:
                if str(ex[attr]).lower() in genes or self.gene_matcher.umatch(str(ex[attr])):
                    examples.append(ex)
            if examples:
                data = Orange.data.Table(examples)
            else:
                data = None
        self.send("Selected Genes", data)
        
    def handle_assync_error(self, *args):
        pass
        
        
class StandardPyItem(QStandardItem):
    def __lt__(self, other):
        my = self.data(Qt.DisplayRole).toPyObject()
        other = other.data(Qt.DisplayRole).toPyObject()
        return my < other
        
    
if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWGeneAtlasTissueExpression()
    data = Orange.data.Table("RUNX1.tab")
    w.show()
    w.set_data(data)
    app.exec_()
    w.saveSettings()
        



"""
<name>Save Data Files</name>
<description>Saves data to selected directory.</description>
<icon>icons/DataFilesSave.png</icon>
<priority>1055</priority>
<contact>Peter Juvan (peter.juvan@fri.uni-lj.si)</contact>
<prototype>1</prototype>
"""

from __future__ import absolute_import

import os, orange

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .OWDataFiles import DataFiles

class OWDataFilesSave(OWWidget):
    settingsList  = ["recentDirs", "selectedDirName"]

    def __init__(self, parent=None, signalManager = None):
        OWWidget.__init__(self, parent, signalManager, "Save Data Files", wantMainArea = 0, resizingEnabled = 1)

        self.inputs = [("Structured Data", DataFiles, self.structuredData, Default)]
        self.outputs = []
        self.dataStructure = None

        # Settings
        self.recentDirs=[]
        self.selectedDirName = "(none)"
        self.loadSettings()

        # GUI
        rfbox = OWGUI.widgetBox(self.controlArea, "Directory", orientation="horizontal", addSpace = True)
        self.dircombo = QComboBox(rfbox)
        rfbox.layout().addWidget(self.dircombo)
        browse = OWGUI.button(rfbox, self, '&...', callback = self.browseDirectory, disabled=0)
        browse.setMaximumWidth(25)
        # info
        box = OWGUI.widgetBox(self.controlArea, "Info", addSpace = True)
        self.infoa = OWGUI.widgetLabel(box, 'No data on input.')
        # Output
        box = OWGUI.widgetBox(self.controlArea, "Output", addSpace = True)
        self.save = OWGUI.button(box, self, '&Save', callback = self.saveData, disabled=1)
        self.adjustSize()
           
        # initial settings            
        self.recentDirs=filter(os.path.exists, self.recentDirs)
        self.setDirlist()
        self.dircombo.setCurrentIndex(0)
        self.resize(300,self.height())

    def structuredData(self, data):
        self.dataStructure = data
        self.save.setDisabled(self.dataStructure == None)
        if self.dataStructure == None:
            self.infoa.setText("No data on input.")
        else:
            self.infoa.setText("New data on input.")
            
    def saveData(self):
        # Saves data into root directory
        rootDir = self.recentDirs[self.dircombo.currentIndex()]
        if rootDir == "(none)" or self.dataStructure == None:
            self.infoa.setText("Select a directory first.")
            return
        # count number of files to save
        n = sum([sum([1 for d in ds]) for (sdn, ds) in self.dataStructure])	
        if n == 0:
            self.infoa.setText("No files to save.")
            return
        pbStep = 100./n
        self.progressBarInit()

        for (subDirName, datasets) in self.dataStructure:
            targetDir = os.path.join(rootDir, subDirName)
            if not os.path.exists(targetDir):
                try:
                    os.mkdir(targetDir)
                except:
                    self.infoa.setText("Could not create target directory: " + targetDir)
                    self.error("Could not create target directory: " + targetDir)

            for data in datasets:
                fname = os.path.join(targetDir, data.name + '.tab')
                orange.saveTabDelimited(fname, data)
                self.progressBarAdvance(pbStep)
        self.infoa.setText("Data saved to %s" % rootDir)
        self.progressBarFinished()

    def browseDirectory(self):
        # displays a file dialog and selects a directory
        if len(self.recentDirs):
            startdir=self.recentDirs[0]
        else:
            startdir ="."
        dirname=str(QFileDialog.getExistingDirectory(self, 'Select Directory to Save into', startdir, QFileDialog.ShowDirsOnly))
        if len(dirname):
            self.addDirToList(dirname)

    def setDirlist(self):
        self.dircombo.clear()
        if len(self.recentDirs):
            for dir in self.recentDirs:
                ### leave out the path
                ##(upperdir,dirname)=os.path.split(dir)
                self.dircombo.insertItem(self.dircombo.count(), dir)
        else:
            self.dircombo.insertItem(0, "(none)")

    def addDirToList(self, dir):
        # Add a directory to the start of the file list. 
        # If it exists, move it to the start of the list
        dir = os.path.normpath(dir)
        if dir in self.recentDirs:
            self.recentDirs.remove(dir)
        self.recentDirs.insert(0, str(dir))
        self.setDirlist()
        self.selectedDirName = dir

    def selectDir(self, n):
        # called when user makes a selection from the drop-down menu
        if self.recentDirs:
            self.loadData(self.recentDirs[n])
            self.addDirToList(self.recentDirs[n])
        else:
            self.loadData("(none)")


if __name__=="__main__":
    a=QApplication(sys.argv)
    ow=OWDataFilesSave()
    ow.show()
    a.exec_()
    ow.saveSettings()



""" Interface to retrieve gene networks from GeneMANIA server

Example::
    
    >>> conn = Connection("http://localhost:8080/genemania")
    >>> net = conn.retrieve(org="3702", ["PHYB", "ELF3", 'COP1", "SPA1", "FUS9"])
    >>> net.save("graph.net")
    >>> net.retrieve(org="3702", genes=["CIP1"], m="bp", r=100).save("CIP1.net")

"""

from __future__ import absolute_import

import urllib2
import urllib
import re
import posixpath
from xml.dom import minidom

import orange

DEFAULT_SERVER = "http://193.2.72.57:8080/genemania"

_TAX_ID_2_INDEX = {"3702": 1,
                   "6239": 2,
                   "7227": 3,
                   "9606": 4,
                   "10090": 5,
                   "4932": 6
                   }
class Connection(object):
    _RE_TOKEN = re.compile(r'<li\s+id\s*=\s*"menu_save"\s*token\s*=\s*"([0-9]+)"><label>Save</label>')
    _RE_NETWORK_TAB = re.compile(r'^<div\s*id\s*=\s*"networks_tab"\s*class\s*=\s*"tab">*?^</div>', re.MULTILINE)
    _RE_NETWORK_GROUP_NAMES = re.compile(r'<div\s*class\s*=\s*"network_name"\s*id\s*=\s*"networkGroupName([0-9]+)"\s*>\s*([a-zA-Z0-9_\- ]+)\s*</div>')
    _RE_NETWORK_NAMES = re.compile(r'<div\s*class\s*=\s*"network_name"\s*id\s*=\s*"networkName([0-9]+)"\s*>\s*([a-zA-Z0-9_\- ]+)\s*</div>')

    def __init__(self, address=DEFAULT_SERVER):
        """ Construct a Connection instance for GeneMANIA server at `address`
        
        :param address: URL address of GeneMANIA server
        :type address: str
        """
        self.address = address
                  
        
    def retrieveXML(self, org="9606", genes=[], m="automatic", r=10, token=None):
        """ Same as `retrieve` but return the network as an xml string
        """
        if token is None:
            page = self.retrieveHtmlPage(org, genes, m, r)
#            query = self._queryPage(org, genes, m, r)
#            stream = urllib2.urlopen(query)
#            page = stream.read()
            match = self._RE_TOKEN.findall(page)
        
            if match:
                token = match[0]
            else:
                raise ValueError("Invalid query. %s" % self._queryPage(org, genes, m, r))
        
        query = self._queryGraph(token)
        stream = urllib2.urlopen(query)
        graph = stream.read()
        self._graph = graph
        return graph
    
    def retrieveHtmlPage(self, org="9606", genes=[], m="automatic", r=10):
        """ Retrieve the HTML page (contains token to retrieve the graph, network descriptions ...)"
        """
        query = self._queryPage(org, genes, m, r)
        stream = urllib2.urlopen(query)
        page = stream.read()
        self._page = page
        return page
    
    def validate(self, org, genes):
        """ Validate gene names for organism. Return a two 
        tuple, one with known and one with unknown genes
        """
        
        organism = _TAX_ID_2_INDEX.get(org, 1)
        genes = "; ".join(genes)
        data = urllib.urlencode([("organism", str(organism)), ("genes", genes)])
        validatorUrl = posixpath.join(self.address, "validator")
        stream = urllib2.urlopen(validatorUrl, data)
        response = stream.read()
        dom = minidom.parseString(response)
        return parseValidationResponse(dom)
        
        
        
    def _queryPage(self, org, genes, m, r):
        return posixpath.join(self.address, "link?o=%s&g=%s&m=%s&r=%i" % (org, "|".join(genes), m, r)) 
    
    
    def _queryGraph(self, token):
        return posixpath.join(self.address, "pages/graph.xhtml?token=%s" % token)
    
    
    def retrieve(self, org, genes, m="automatic", r=10):
        """ Retrieve :class:`Orange.network.Graph` instance representing the network for
        the query.
        
        :param org: NCBI taxonomy identifier (A. thaliana=3702, C. elegans=6239,
                    D. melanogaster=7227, H. sapiens=9606, M. musculus=10090
                    S. cerevisiae=4932)
        :type org: str
        
        :param genes: query genes
        :type genes: list
        
        :param m: network combining method; must be one of the following:
                    * "automatic_relevance": Assigned based on query genes
                    * "automatic": Automatically selected weighting method
                       (Default)
                    * "bp": biological process based
                    * "mf": molecular function based
                    * "cc": cellular component based
                    * "average": equal by data type
                    * "average_category: equal by network
        :type m: str
        
        :param r: the number of results generated by GeneMANIA (must be in 
                  range 1..100
        :type r: int
        
        .. note:: See `http://193.2.72.57:8080/genemania/pages/help.jsf#section/link`_ for
            more details on the parameters. 
        """
        xml = self.retrieveXML(org, genes, m, r)
        dom = minidom.parseString(xml)
        graph = parse(dom)
        return graph
    
        
    
def parse(DOM):
    """ Parse the graph DOM as returned from geneMANIA server and return
    an :class:`Orange.network.Graph` instance.
    
    """
    nodes = DOM.getElementsByTagName("node")
    edges = DOM.getElementsByTagName("edge")
    from collections import defaultdict
    graphNodes = {}
    graphEdges = defaultdict(list)
    
    def parseAttributes(element):
        return dict([(key, value) for key, value in element.attributes.items()])
    
    def parseText(element):
        text = u""
        for el in element.childNodes:
            if isinstance(el, minidom.Text):
                text += el.wholeText
        return text
                
    def parseData(node):
        data = node.getElementsByTagName("data")
        parsed = {}
        for el in data:
            attrs = parseAttributes(el)
            key = attrs["key"]
            parsed[key] = parseText(el)
        return parsed
    
    for node in nodes:
        attrs = parseAttributes(node)
        id = attrs["id"]
        data = parseData(node)
        graphNodes[id] = data
    
    for edge in edges:
        attrs = parseAttributes(edge)
        source, target = attrs["source"], attrs["target"]
        data = parseData(edge)
        graphEdges[source, target].append(data)
        
    allData = reduce(list.__add__, graphEdges.values(), [])
    edgeTypes = set([int(data["networkGroupId"]) for data in allData])
    groupId2int = dict(zip(edgeTypes, range(len(edgeTypes))))
    groupId2groupCode = dict([(int(data["networkGroupId"]), str(data["networkGroupCode"])) for data in allData])
    graphNode2nodeNumber = dict(zip(graphNodes, range(len(graphNodes))))
    
    import Orange
    graph = Orange.network.Graph()
    for id, data in graphNodes.items():
        graph.add_node(graphNode2nodeNumber[id],
                       original_id=str(id),
                       symbol=data["symbol"],
                       score=float(data["score"]))
         
    graph.add_nodes_from(sorted(graphNode2nodeNumber.values()))
    
    edgeWeights = []
    for (source, target), edge_data in graphEdges.items():
        edgesDefined = [None] * len(edgeTypes)
        for data in edge_data:
            networkGroupId = int(data["networkGroupId"])
            edgeInd = groupId2int[networkGroupId]
            edgesDefined[edgeInd] = float(data["weight"])
            graph.add_edge(graphNode2nodeNumber[source], 
                           graphNode2nodeNumber[target],
                           weight=float(data["weight"]),
                           networkGroupId=networkGroupId)
            
        edgesDefined = [0 if w is None else w for w in edgesDefined]
        edgeWeights.append(edgesDefined)
        
        
    nodedomain = orange.Domain([orange.StringVariable("label"),
                                orange.StringVariable("id"),
                                orange.FloatVariable("score"),
                                orange.StringVariable("symbol"),
                                orange.StringVariable("go"),
                                orange.EnumVariable("source", values=["true", "false"])], None)
    
    edgedomain = orange.Domain([orange.FloatVariable("u"),
                                orange.FloatVariable("v")] +\
                               [orange.FloatVariable("weight_%s" % groupId2groupCode[id]) for id in edgeTypes],
                               None)
    
    node_items = graphNodes.items()
    node_items = sorted(node_items, key=lambda t: graphNode2nodeNumber[t[0]])
    
    nodeitems = orange.ExampleTable(nodedomain,
                  [[str(node["symbol"]), str(id), float(node["score"]),
                    str(node["symbol"]), str(node["go"]), str(node["source"])]\
                     for id, node in node_items])
    
    edgeitems = orange.ExampleTable(edgedomain,
                  [[str(graphNode2nodeNumber[source] + 1), 
                    str(graphNode2nodeNumber[target] + 1)] + weights \
                   for ((source, target), _), weights in zip(graphEdges.items(), edgeWeights)])
        
    graph.set_items(nodeitems)
    graph.set_links(edgeitems)
    
    return graph

def parseValidationResponse(dom):
    def getData(node):
        data = []
        for c in node.childNodes:
            if c.nodeType == node.TEXT_NODE:
                data.append(c.data)
                
        return " ".join([d.strip() for d in data])
        
    def getStrings(node):
        strings = []
        for string in node.getElementsByTagName("string"):
            strings.append(getData(string))
        return strings
    errorCode = dom.getElementsByTagName("errorCode")[0]
    errorCode = getData(errorCode)
    invalidSymbols = getStrings(dom.getElementsByTagName("invalidSymbols")[0])
    geneIds = getStrings(dom.getElementsByTagName("geneIds")[0])
    
    return errorCode, invalidSymbols, geneIds
    

from HTMLParser import HTMLParser

class NetworkGroup(object):
    """ Network group descriptor
    """
    def __init__(self):
        self.weight = ""
        self.networks = []
        self.name = ""
        self.id = ""


class Network(object):
    """ Source network descriptor
    """
    
    def __init__(self):
        self.weight = ""
        self.name = ""
        self.id = ""
        self.description = ""
        
        
class _NetworkTabParser(HTMLParser):
    """ Parses the "Network" tab from the GeneMANIA HTML pages 
    """
    _RE_GROUP_ID = re.compile(r"networkGroup(\d+)")
    _RE_GROUP_WEIGHT_ID = re.compile(r"networkGroupWeight(\d+)")
    _RE_GROUP_NAME_ID = re.compile(r"networkGroupName(\d+)")
    
    _RE_NETWORK_ID = re.compile(r"network(\d+)")
    _RE_NETWORK_WEIGHT_ID = re.compile(r"networkWeight(\d+)")
    _RE_NETWORK_NAME_ID = re.compile(r"networkName(\d+)")
    _RE_NETWORK_DESCRIPTION_ID = re.compile("networkDescription(\d+)")
    
    
    def __init__(self, *args, **kwargs):
        HTMLParser.__init__(self)
        self.networkGroups = []
        self.networks = {}
        
        self.currentGroup = None
        self.currentNetwork = None
        
        self.data_handler = None
        
    def handle_start_group(self, tag, attrs):
        """ Handle '<li class=... id="networkGroup%i">'
        """
        self.currentGroup = NetworkGroup()
        self.currentGroup.id = attrs.get("id")
        
        self.networkGroups.append(self.currentGroup)
        
        
    def handle_start_group_weight(self, tag, attrs):
        """ Handle '<span tooltip="..." id="networkGroupWeight%i">'
        """
        self.data_handler = self.handle_group_weight_data
        
    def handle_group_weight_data(self, data):
        self.currentGroup.weight += data
        
    def handle_end_group_weight(self, tag):
        self.data_handler = None
        
    def handle_start_group_name(self, tag, attrs):
        """ Handle '<div class="network_name" id="networkGroupName%i">'
        """
        self.data_handler = self.handle_group_name_data
        
    def handle_group_name_data(self, data):
        self.currentGroup.name += data
        
    def handle_start_network(self, tag, attrs):
        """ Handle '<li class="checktree_network" id="network%i">'
        """
        self.currentNetwork = Network()
        self.currentNetwork.id = attrs.get("id")
        
        self.currentGroup.networks.append(self.currentNetwork)
        
    def handle_start_network_weight(self, tag, attrs):
        """ Handle '<span tooltip="..." id="networkWeight%i">'
        """
        self.data_handler = self.handle_network_weight_data
        
    def handle_network_weight_data(self, data):
        self.currentNetwork.weight += data
        
    def handle_start_network_name(self, tag, attrs):
        """ Handle '<div class="network_name" id="networkName%i">'
        """
        self.data_handler = self.handle_network_name_data
        
    def handle_network_name_data(self, data):
        self.currentNetwork.name += data
        
    def handle_start_network_description(self, tag, attrs):
        """ Handle '<div class="text" id="networkDescription%i">'
        """
        self.data_handler = self.handle_network_description_data
        
    def handle_network_description_data(self, data):
        self.currentNetwork.description += data
        
    def handle_data(self, data):
        if self.data_handler:
            self.data_handler(data)
    
    def handle_starttag(self, tag, attrs):
        attrs = dict(attrs)
        if tag == "li" and self._RE_GROUP_ID.search(attrs.get("id", "")):
            self.handle_start_group(tag, attrs)
        elif tag == "span" and self._RE_GROUP_WEIGHT_ID.search(attrs.get("id", "")):
            self.handle_start_group_weight(tag, attrs)
        elif tag == "div" and self._RE_GROUP_NAME_ID.search(attrs.get("id", "")):
            self.handle_start_group_name(tag, attrs)
        elif tag == "li" and self._RE_NETWORK_ID.search(attrs.get("id", "")):
            self.handle_start_network(tag, attrs)
        elif tag == "span" and self._RE_NETWORK_WEIGHT_ID.search(attrs.get("id", "")):
            self.handle_start_network_weight(tag, attrs)
        elif tag == "div" and self._RE_NETWORK_NAME_ID.search(attrs.get("id", "")):
            self.handle_start_network_name(tag, attrs)
        elif tag == "div" and self._RE_NETWORK_DESCRIPTION_ID.search(attrs.get("id", "")):
            self.handle_start_network_description(tag, attrs)
        else:
            HTMLParser.handle_starttag(self, tag, attrs)
            
    def handle_endtag(self, tag):
        self.data_handler = None
            

def parsePage(html):
    parser = _NetworkTabParser()
    parser.feed(html)
    return parser.networkGroups
    

def retrieve(org=None, genes=[], m="automatic", r=10):
    """ A helper function, same as Connection().retrive(*args, **kwargs)
    """
    return Connection().retrieve(org, genes, m, r)


"""
======================
PPI Database interface
======================

"""


import sqlite3
import csv
import os
import posixpath

from contextlib import contextmanager
import StringIO

@contextmanager
def finishing(obj):
    """ Calls obj.finish() on context exit.
    """
    yield obj
    obj.finish()

def guess_size(fileobj):
    try:
        if isinstance(fileobj, file):
            return os.fstat(fileobj.fileno()).st_size
        elif isinstance(fileobj, StringIO.StringIO):
            pos = fileobj.tell()
            fileobj.seek(0, 2)
            length = fileobj.tell() - pos
            fileobj.seek(pos, 0)
            return length
        elif isinstance(fileobj, urllib.addinfourl):
            length = fileobj.headers.get("content-length", None)
            return length
    except Exception, ex:
        pass


def copyfileobj(src, dst, buffer=2**10, content_len=None, progress=None):
    count = 0
    if content_len is None:
        content_len = guess_size(src) or sys.maxint
    while True:
        data = src.read(buffer)
        dst.write(data)
        count += len(data)
        if progress:
            progress(100.0 * count / content_len)
        if not data:
            break
            
            
def wget(url, directory=".", dst_obj=None, progress=None):
    """
    .. todo:: Move to Orange.misc
    
    """
    stream = urllib2.urlopen(url)
    length = stream.headers.get("content-length", None)
    if length is None:
        length = sys.maxint
    else:
        length = int(length)
    
    basename = posixpath.basename(url)
        
    if dst_obj is None:
        dst_obj = open(os.path.join(directory, basename), "wb")
    
    if progress == True:
        from Orange.utils import ConsoleProgressBar
        progress = ConsoleProgressBar("Downloading %r." % basename)
        with finishing(progress):
            copyfileobj(stream, dst_obj, buffer=2**10, content_len=length,
                        progress=progress)
    else:
        copyfileobj(stream, dst_obj, buffer=2**10, content_len=length,
                    progress=progress)
    
from . import obiPPI
from Orange.orng import orngServerFiles

from . import obiTaxonomy
from collections import namedtuple
from operator import itemgetter
from Orange.utils import lru_cache

GENE_MANIA_INTERACTION_FIELDS = \
    ["gene_a", "gene_b", "weight", "network_name",
     "network_group", "source", "pubmed_id"]
     
GeneManiaInteraction = namedtuple("GeneManiaInteraction",
                                  field_names=GENE_MANIA_INTERACTION_FIELDS
                                 )

import weakref
class Internalizer(object):
    """ A class that acts as the python builtin function ``intern``,
    for as long as it is alive.
    
    .. note:: This is for memory optimization only, it does not affect 
        dict lookup speed.
    
    """
    def __init__(self):
        self._intern_dict = {}
        
    def __call__(self, obj):
        return self._intern_dict.setdefault(obj, obj)
    
class GeneManiaDatabase(obiPPI.PPIDatabase):
    DOMAIN = "PPI"
    SERVER_FILE = "gene-mania-{taxid}.sqlite"
    
    TAXID2NAME = ""
    
    # DB schema
    SCHEMA = """
    table: `genes`
        - `internal_id`: int (pk)
        - `gene_name`: text (preferred name)
        
    table: `synonyms`:
        - `internal_id: int (foreign key `genes.internal_id`)
        - `synonym`: text
        - `source_id`: int
        
    table: `source`:
        - `source_id`: int
        - `source_name`: text
        
    table: `links`:
        - `gene_a`: int (fk `genes.internal_key`)
        - `gene_b`: int (fk `genes.internal_key`)
        - `network_id`: (fk `networks.network_id`)
        - `weight`: real
        
    table: `networks`:
        - `network_id`: int
        - `network_name`: text
        - `network_group`: text
        - `source`: text
        - `pubmed_id`: text
        
    view: `links_annotated`:
        - `gene_name_a`
        - `gene_name_b`
        - `network_name`
        - `network_group`
        - `weight`
        
    """
    
    def __init__(self, taxid):
        self.taxid = taxid
        
    @classmethod
    def common_taxids(self):
        return ["3702", "6239", "7227", "9606", "10090", "10116", "4932"]
    
    def organisms(self):
        """ Return all organism taxids contained in this database.
        
        .. note:: a single taxid is returned (the one used at
            instance initialization)   
        
        """
        return [self.taxid]
    
    def ids(self, taxid=None):
        """ Return all primary ids for `taxid`.
        """
        if taxid is None:
            taxids = self.organisms()
            return reduce(list.__add__, map(self.ids, taxids), [])
        
        con = self._db(taxid)
        cur = con.execute("""\
            SELECT gene_name FROM genes
            """)
        return map(itemgetter(0), cur)
        
    def synonyms(self, id):
        """ Return a list of synonyms for primary `id`.
        """
        con = self._db(self.taxid)
        cur = con.execute("""\
            SELECT synonyms.synonym
            FROM synonyms NATURAL LEFT JOIN genes
            WHERE genes.gene_name=?
            """, (id,))
        return map(itemgetter(0), cur)
        
    def all_edges(self, taxid=None):
        """ Return a list of all edges.
        """
        con = self._db(self.taxid)
        cur = con.execute("""
            SELECT links.gene_a, links.gene_b, links.weight
            FROM links""")
        id_to_name = self._gene_id_to_name()
        return [(id_to_name[r[0]], id_to_name[r[1]], r[2]) \
                for r in cur]
        
    def all_edges_annotated(self, taxid=None):
        """ Return a list of all edges with all available annotations
        """
        con = self._db(self.taxid)
        cur = con.execute("""\
            SELECT links.gene_a, links.gene_b, links.weight, links.network_id
            FROM links""")
        gene_to_name = self._gene_id_to_name()
        network_to_description = self._network_id_to_description()
        res = []
        for gene_a, gene_b, w, n_id in cur:
            n_desc = network_to_description[n_id]
            
            res.append(GeneManiaInteraction(gene_to_name[gene_a],
                            gene_to_name[gene_b], w, *n_desc))
        return res
        
    def edges(self, id1):
        """ Return all edges for primary id `id1`.
        """        
        con = self._db(self.taxid)
        cur = con.execute("""\
            SELECT genes1.gene_name, genes2.gene_name, links.weight
            FROM genes AS genes1  
                JOIN links
                    ON genes1.internal_id=links.gene_a
                JOIN genes AS genes2
                    ON genes2.internal_id=links.gene_b
            WHERE genes1.gene_name=?
            """, (id1,))
        res = cur.fetchall()
        cur = con.execute("""\
            SELECT genes1.gene_name, genes2.gene_name, links.weight
            FROM genes AS genes1  
                JOIN  links
                    ON genes1.internal_id=links.gene_a
                JOIN genes AS genes2
                    ON genes2.internal_id=links.gene_b
            WHERE genes2.gene_name=?
            """, (id1,))
        res += cur.fetchall()
        
        return res
    
    def edges_annotated(self, id=None):
        """ Return a list of annotated edges for primary `id` 
        """
        con = self._db(self.taxid)
        cur = con.execute("""\
            SELECT genes1.gene_name, genes2.gene_name, links.weight,
                   networks.network_name, networks.network_group,
                   networks.source, networks.pubmed_id
            FROM genes AS genes1
                JOIN  links
                    ON genes1.internal_id=links.gene_a
                JOIN genes AS genes2
                    ON genes2.internal_id=links.gene_b
                NATURAL JOIN networks
            WHERE genes1.gene_name=?
            """, (id,))
        res = cur.fetchall()
        cur = con.execute("""\
            SELECT genes1.gene_name, genes2.gene_name, links.weight,
                   networks.network_name, networks.network_group,
                   networks.source, networks.pubmed_id
            FROM genes AS genes1
                JOIN links
                    ON genes1.internal_id=links.gene_a
                JOIN genes AS genes2
                    ON genes2.internal_id=links.gene_b
                NATURAL JOIN networks
            WHERE genes2.gene_name=?
            """, (id,))
        res += cur.fetchall()
        return [GeneManiaInteraction(*r) for r in res]
    
    def search_id(self, name, taxid=None):
        """ Search the database for gene name. Return a list of matching 
        primary ids. Use `taxid` to limit the results to a single organism.
        
        """
        con = self._db(self.taxid)
        cur = con.execute("""\
            SELECT genes.gene_name
            FROM genes NATURAL JOIN synonyms
            WHERE synonyms.synonym=? 
            """, (name,))
        return map(itemgetter(0), cur)
        
    def _db(self, taxid=None):
        """ Return an open sqlite3.Connection object.  
        """
        taxid = taxid or self.taxid
        filename = orngServerFiles.localpath_download("PPI",
                            self.SERVER_FILE.format(taxid=taxid))
        if not os.path.exists(filename):
            raise ValueError("Database is missing.")
        
        return sqlite3.connect(filename)
    
    @lru_cache(maxsize=1)
    def _gene_id_to_name(self):
        """ Return a dictionary mapping internal gene ids to 
        primary gene identifiers.
        
        """
        con = self._db(self.taxid)
        cur = con.execute("SELECT * FROM genes")
        return dict(cur)
    
    @lru_cache(maxsize=1)
    def _network_id_to_description(self):
        """ Return a dictionary mapping internal network ids
        to (name, group, source, pubmed id).
         
        """
        con = self._db(self.taxid)
        cur = con.execute("SELECT * FROM networks")
        return dict((t[0], t[1:]) for t in cur)
    
    #####################################
    # Data download and DB initialization
    #####################################
     
    @classmethod
    def download_data(cls, taxid=None, progress_callback=None):
        """ Download the data for ``taxid`` from the GeneMANIA
        website and initialize the local database.
        
        """
        import tarfile
        
        baseurl = "http://genemania.org/data/current/"
        directory = orngServerFiles.localpath("PPI")
        if taxid is None:
            taxid = cls.common_taxids()
        
        if isinstance(taxid, (list, tuple)):
            taxids = taxid
        else:
            taxids = [taxid]
        for taxid in taxids:
            name = obiTaxonomy.name(taxid)
            name = name.replace(" ", "_")
            
            if progress_callback is None:
                progress = True #orngServerFiles.ConsoleProgressBar("Downloading %r." % filename)
            else:
                progress = progress_callback
            
            filename = name + ".tgz"
            url = baseurl + "networks/" + filename    
            wget(url, directory=directory, progress=progress)
            
            tgz_filename = os.path.join(directory, filename)    
            tgz = tarfile.open(tgz_filename)
            tgz.extractall(directory)
            
            filename = name + ".COMBINED.tgz"
            url = baseurl + "precombined/" + filename
            wget(url, directory=directory, progress=progress)
            
            tgz_filename = os.path.join(directory, filename)
            tgz = tarfile.open(tgz_filename)
            tgz.extractall(directory)
        
            cls.init_db([taxid])
        
    @classmethod
    def init_db(cls, taxid=None):
        """ Init the local data base.
        """
        from functools import partial
        directory = orngServerFiles.localpath("PPI")
        pjoin = partial(os.path.join, directory)
        if taxid is None:
            taxid = cls.common_taxids()
            
        if isinstance(taxid, (list, tuple)):
            for tid in taxid:
                cls.init_db(tid)
            return
                
        if not isinstance(taxid, basestring):
            raise ValueError("wrong taxid")
            
#        taxid = taxids
        name = obiTaxonomy.name(taxid).replace(" ", "_")
        networks = csv.reader(open(pjoin(name, "networks.txt")), delimiter="\t")
        networks.next() # Header
        networks = list(networks)
        
        database = pjoin(cls.SERVER_FILE.format(taxid=taxid))
        with sqlite3.connect(database) as con:
            con.execute("""DROP TABLE IF EXISTS genes""")
            con.execute("""DROP TABLE IF EXISTS synonyms""")
            con.execute("""DROP TABLE IF EXISTS source""")
            con.execute("""DROP TABLE IF EXISTS links""")
            con.execute("""DROP TABLE IF EXISTS networks""")
            
            con.execute("""DROP INDEX IF EXISTS genes_index""")
            con.execute("""DROP INDEX IF EXISTS links_index_a""")
            con.execute("""DROP INDEX IF EXISTS links_index_b""")
            
            con.execute("""\
                CREATE TABLE networks 
                    (network_id INTEGER PRIMARY KEY ASC AUTOINCREMENT,
                     network_name TEXT,
                     network_group TEXT,
                     source TEXT,
                     pubmed_id TEXT
                    )""")
            
            con.executemany("""\
                INSERT INTO networks
                VALUES (?, ?, ?, ?, ?)""", [(i, r[2], r[1], r[3], r[4]) \
                                        for i, r in enumerate(networks)])
            
            con.execute("""\
                CREATE TABLE genes
                    (internal_id INTEGER PRIMARY KEY ASC AUTOINCREMENT,
                     gene_name TEXT
                    )""")
            
            identifiers = csv.reader(open(pjoin(name, "identifier_mappings.txt"), "rb"),
                                    delimiter="\t")
            identifiers.next() # skip header
            identifiers = list(identifiers)
            genes = sorted(set(r[0] for r in identifiers))
            sources = sorted(set(r[2] for r in identifiers))
            
            con.executemany("""\
                INSERT INTO genes
                VALUES (?, ?)""", enumerate(genes))
            
            con.execute("""\
            CREATE TABLE source
                (source_id INTEGER PRIMARY KEY ASC AUTOINCREMENT,
                 source_name TEXT
                )""")
            
            con.executemany("""\
                INSERT INTO source
                VALUES (?, ?)""", enumerate(sources))
            
            con.execute("""\
                CREATE TABLE synonyms
                    (internal_id INTEGER REFERENCES genes (internal_id),
                     synonym TEXT,
                     source_id INT REFERENCES source (source_id)
                    )""")
            
            gene_to_id = dict((g, i) for i, g in enumerate(genes))
            source_to_id = dict((s, i) for i, s in enumerate(sources))
            con.executemany("""\
                INSERT INTO synonyms
                VALUES (?, ?, ?)""", [(gene_to_id[r[0]], r[1], source_to_id[r[2]])\
                                       for r in identifiers])
            
            con.execute("""\
                CREATE TABLE links
                    (gene_a INTEGER REFERENCES genes (internal_id),
                     gene_b INTEGER REFERENCES genes (internal_id),
                     network_id INTEGER REFERENCES networks (network_id),
                     weight REAL
                     -- , PRIMARY KEY (gene_a, gene_b, network_id)
                    )""")
            
            for i, (filename, group, _, _, _) in enumerate(networks):
                nf  = open(pjoin(name, filename), "rb")
                interactions = csv.reader(nf, delimiter="\t")
                interactions.next() # skip header
                con.executemany("""\
                    INSERT INTO links 
                    VALUES (?, ?, ?, ?)""",
                    [(gene_to_id[r[0]], gene_to_id[r[1]], i, float(r[2])) \
                     for r in interactions]
                )
                
            # Add special combined network entry
            combined_id = len(networks)
            con.execute("""\
                INSERT INTO networks
                VALUES (?, ?, ?, ?, ?)""", 
                (combined_id, "BP_COMBINING", "COMBINED", "GeneMANIA", ""))
            
            # Add the combined network links.
            combined = open(pjoin(name + ".COMBINED", "COMBINED.DEFAULT_NETWORKS.BP_COMBINING.txt"), "rb")
            combined = csv.reader(combined, delimiter="\t")
            combined.next()
            con.executemany("""\
                INSERT INTO links
                VALUES (?, ?, ?, ?)""",
                    ((gene_to_id[r[0]], gene_to_id[r[1]], combined_id, float(r[2])) \
                     for r in combined))
            
            
            con.execute("""
                CREATE VIEW IF NOT EXISTS links_annotated
                AS SELECT genes1.gene_name AS gene_name_a,
                          genes2.gene_name AS gene_name_b,
                          links.weight,
                          networks.network_name,
                          networks.network_group,
                          networks.source,
                          networks.pubmed_id
                   FROM  genes AS genes1
                        JOIN links 
                              ON genes1.internal_id=links.gene_a
                        JOIN genes AS genes2
                              ON links.gene_b=genes2.internal_id
                        JOIN networks
                              ON links.network_id=networks.network_id
                    """)
            
            
            con.execute("""\
                CREATE INDEX IF NOT EXISTS genes_index ON genes (gene_name)
                """)
            con.execute("""\
                CREATE INDEX IF NOT EXISTS links_index_a ON links (gene_a)
                """)
            con.execute("""\
                 CREATE INDEX IF NOT EXISTS links_index_b ON links (gene_b)
                """)
        
            
if __name__ == "__main__":
    retrieve("9606", [ 'MRE11A', 'RAD51', 'MLH1', 'MSH2', 'DMC1', 'RAD51AP1', 'RAD50', 'MSH6', 'XRCC3', 'PCNA', 'XRCC2' ])



from __future__ import absolute_import

from collections import defaultdict
import random
import time

import numpy

import orange

from . import obiGeneSets
from .obiExpression import *

"""
Gene set enrichment analysis.

Author: Marko Toplak
"""

def iset(data):
    """
    Is data orange.ExampleTable?
    """
    return isinstance(data, orange.ExampleTable)

def issequencens(x):
    "Is x a sequence and not string ? We say it is if it has a __getitem__ method and is not string."
    return hasattr(x, '__getitem__') and not isinstance(x, basestring)

def mean(l):
    return float(sum(l))/len(l)

def rankingFromOrangeMeas(meas):
    """
    Creates a function that sequentally ranks all attributes and returns
    results in a list. Ranking function is build out of 
    orange.MeasureAttribute.
    """
    return lambda d: [ meas(i,d) for i in range(len(d.domain.attributes)) ]

def orderedPointersCorr(lcor):
    """
    Return a list of integers: indexes in original
    lcor. Elements in the list are ordered by
    their lcor[i] value. Higher correlations first.
    """
    ordered = [ (i,a) for i,a in enumerate(lcor) ] #original pos + correlation
    ordered.sort(lambda x,y: cmp(y[1],x[1])) #sort by correlation, descending
    ordered = nth(ordered, 0) #contains positions in the original list
    return ordered

def enrichmentScoreRanked(subset, lcor, ordered, p=1.0, rev2=None):
    """
    Input data and subset. 
    
    subset: list of attribute indices of the input data belonging
        to the same set.
    lcor: correlations with class for each attribute in a list. 

    Returns enrichment score on given data.

    This implementation efficiently handles "sparse" genesets (that
    cover only a small subset of all genes in the dataset).
    """

    #print lcor

    subset = set(subset)

    if rev2 == None:
        def rev(l):
            return numpy.argsort(l)
        rev2 = rev(ordered)

    #add if gene is not in the subset
    notInA = -(1. / (len(lcor)-len(subset)))
    #base for addition if gene is in the subset
    cors = [ abs(lcor[i])**p for i in subset ]
    sumcors = sum(cors)

    #this should not happen
    if sumcors == 0.0:
        return (0.0, None)
    
    inAb = 1./sumcors

    ess = [0.0]
    
    map = {}
    for i in subset:
        orderedpos = rev2[i]
        map[orderedpos] = inAb*abs(lcor[i]**p)
        
    last = 0

    maxSum = minSum = csum = 0.0

    for a,b in sorted(map.items()):
        diff = a-last
        csum += notInA*diff
        last = a+1
        
        if csum < minSum:
            minSum = csum
        
        csum += b

        if csum > maxSum:
            maxSum = csum

    #finish it
    diff = (len(ordered))-last
    csum += notInA*diff

    if csum < minSum:
        minSum = csum

    #print "MY", (maxSum if abs(maxSum) > abs(minSum) else minSum)

    """
    #BY DEFINITION
    print "subset", subset

    for i in ordered:
        ess.append(ess[-1] + \
            (inAb*abs(lcor[i]**p) if i in subset else notInA)
        )
        if i in subset:
            print ess[-2], ess[-1]
            print i, (inAb*abs(lcor[i]**p))

    maxEs = max(ess)
    minEs = min(ess)
    
    print "REAL", (maxEs if abs(maxEs) > abs(minEs) else minEs, ess[1:])

    """
    return (maxSum if abs(maxSum) > abs(minSum) else minSum, [])

#from mOrngData
def shuffleAttribute(data, attribute, locations):
    """
    Destructive!
    """
    attribute = data.domain[attribute]
    l = [None]*len(data)
    for i in range(len(data)):
        l[locations[i]] = data[i][attribute]
    for i in range(len(data)):
        data[i][attribute] = l[i]

def shuffleClass(datai, rands=0):
    """
    Returns a dataset with values of class attribute randomly shuffled.
    If multiple dataset are on input shuffle them all with the same random seed.
    """
    def shuffleOne(data):
        rand = random.Random(rands)
        d2 = orange.ExampleTable(data.domain, data)
        locations = range(len(data))
        rand.shuffle(locations)
        shuffleAttribute(d2, d2.domain.classVar, locations)
        return d2

    if iset(datai):
        return shuffleOne(datai)
    else:
        return [ shuffleOne(data) for data in datai ]

def shuffleList(l, rand=random.Random(0)):
    """
    Returns a copy of a shuffled input list.
    """
    import copy
    l2 = copy.copy(l)
    rand.shuffle(l2)
    return l2

def shuffleAttributes(data, rand=random.Random(0)):
    """
    Returns a dataset with a new attribute order.
    """
    natts = shuffleList(list(data.domain.attributes), rand)
    dom2 = orange.Domain(natts, data.domain.classVar)
    d2 = orange.ExampleTable(dom2, data)
    return d2

def gseapval(es, esnull):
    """
    From article (PNAS):
    estimate nominal p-value for S from esnull by using the positive
    or negative portion of the distribution corresponding to the sign 
    of the observed ES(S).
    """
    
    try:
        if es < 0:
            return float(len([ a for a in esnull if a <= es ]))/ \
                len([ a for a in esnull if a < 0])    
        else: 
            return float(len([ a for a in esnull if a >= es ]))/ \
                len([ a for a in esnull if a >= 0])
    except:
        return 1.0


def enrichmentScore(data, subset, rankingf):
    """
    Returns enrichment score and running enrichment score.
    """
    lcor = rankingf(data)
    ordered = orderedPointersCorr(lcor)
    es,l = enrichmentScoreRanked(subset, lcor, ordered)
    return es,l

def gseaE(data, subsets, rankingf=None, \
        n=100, permutation="class", **kwargs):
    """
    Run GSEA algorithm on an example table.

    data: orange example table. 
    subsets: list of distinct subsets of data.
    rankingf: function that returns correlation to class of each 
        variable.
    n: number of random permutations to sample null distribution.
    permutation: "class" for permutating class, else permutate attribute 
        order.

    """

    if not rankingf:
        rankingf=rankingFromOrangeMeas(MA_signalToNoise())

    enrichmentScores = []
 
    lcor = rankingf(data)
    #print lcor

    ordered = orderedPointersCorr(lcor)

    def rev(l):
        return numpy.argsort(l)

    rev2 = rev(ordered)

    for subset in subsets:
        es = enrichmentScoreRanked(subset, lcor, ordered, rev2=rev2)[0]
        enrichmentScores.append(es)

    runOptCallbacks(kwargs)

    #print "PERMUTATION", permutation

    enrichmentNulls = [ [] for a in range(len(subsets)) ]

    for i in range(n):

        if permutation == "class":
            d2 = shuffleClass(data, 2000+i) #fixed permutation
            r2 = rankingf(d2)

        else:
            r2 = shuffleList(lcor, random.Random(2000+i))

        ordered2 = orderedPointersCorr(r2)
        rev22 = rev(ordered2)
        for si,subset in enumerate(subsets):
            esn = enrichmentScoreRanked(subset, r2, ordered2, rev2=rev22)[0]
            enrichmentNulls[si].append(esn)

        runOptCallbacks(kwargs)

    return gseaSignificance(enrichmentScores, enrichmentNulls)


def runOptCallbacks(rargs):
    if "callback" in rargs:
        try:
            [ a() for a in rargs["callback"] ]
        except:
            rargs["callback"]()
            

def gseaR(rankings, subsets, n=100, **kwargs):
    """
    """

    if "permutation" in kwargs:
        if kwargs["permutation"] == "class":
            raise Exception("Only gene permutation possible")

    enrichmentScores = []
 
    ordered = orderedPointersCorr(rankings)
    
    def rev(l):
        return numpy.argsort(l)

    rev2 = rev(ordered)

    for subset in subsets:

        es = enrichmentScoreRanked(subset, rankings, ordered, rev2=rev2)[0]
        enrichmentScores.append(es)
    
    runOptCallbacks(kwargs)

    enrichmentNulls = [ [] for a in range(len(subsets)) ]

    for i in range(n):
        
        r2 = shuffleList(rankings, random.Random(2000+i))
        ordered2 = orderedPointersCorr(r2)
        rev22 = rev(ordered2)

        for si,subset in enumerate(subsets):

            esn = enrichmentScoreRanked(subset, r2, ordered2, rev2=rev22)[0]
            enrichmentNulls[si].append(esn)

        runOptCallbacks(kwargs)

    return gseaSignificance(enrichmentScores, enrichmentNulls)


def gseaSignificance(enrichmentScores, enrichmentNulls):

    #print enrichmentScores

    import time

    tb1 = time.time()

    enrichmentPVals = []
    nEnrichmentScores = []
    nEnrichmentNulls = []

    for i in range(len(enrichmentScores)):
        es = enrichmentScores[i]
        enrNull = enrichmentNulls[i]
        #print es, enrNull

        enrichmentPVals.append(gseapval(es, enrNull))

        #normalize the ES(S,pi) and the observed ES(S), separetely rescaling
        #the positive and negative scores by divident by the mean of the 
        #ES(S,pi)

        #print es, enrNull

        def normalize(s):
            try:
                if s == 0:
                    return 0.0
                if s >= 0:
                    meanPos = mean([a for a in enrNull if a >= 0])
                    #print s, meanPos
                    return s/meanPos
                else:
                    meanNeg = mean([a for a in enrNull if a < 0])
                    #print s, meanNeg
                    return -s/meanNeg
            except:
                return 0.0 #return if according mean value is uncalculable


        nes = normalize(es)
        nEnrichmentScores.append(nes)
        
        nenrNull = [ normalize(s) for s in enrNull ]
        nEnrichmentNulls.append(nenrNull)
 

    #print "First part", time.time() - tb1

    #FDR computation
    #create a histogram of all NES(S,pi) over all S and pi
    vals = reduce(lambda x,y: x+y, nEnrichmentNulls, [])


    def shorten(l, p=10000):
        """
        Take each len(l)/p element, if len(l)/p >= 2.
        """
        e = len(l)/p
        if e <= 1:
            return l
        else:
            return [ l[i] for i in xrange(0, len(l), e) ]

    #vals = shorten(vals) -> this can speed up second part. is it relevant TODO?

    """
    Use this null distribution to compute an FDR q value, for a given NES(S) =
    NES* >= 0. The FDR is the ratio of the percantage of all (S,pi) with
    NES(S,pi) >= 0, whose NES(S,pi) >= NES*, divided by the percentage of
    observed S wih NES(S) >= 0, whose NES(S) >= NES*, and similarly if NES(S)
    = NES* <= 0.
    """

    nvals = numpy.array(sorted(vals))
    nnes = numpy.array(sorted(nEnrichmentScores))

    #print "LEN VALS", len(vals), len(nEnrichmentScores)

    fdrs = []

    import operator

    for i in range(len(enrichmentScores)):

        nes = nEnrichmentScores[i]

        """
        #Strighfoward but slow implementation follows in comments.
        #Useful as code description.
        
        if nes >= 0:
            op0 = operator.ge
            opn = operator.ge
        else:
            op0 = operator.lt
            opn = operator.le

        allPos = [a for a in vals if op0(a,0)]
        allHigherAndPos = [a for a in allPos if opn(a,nes) ]

        nesPos = [a for a in nEnrichmentScores if op0(a,0) ]
        nesHigherAndPos = [a for a in nesPos if opn(a,nes) ]

        top = len(allHigherAndPos)/float(len(allPos)) #p value
        down = len(nesHigherAndPos)/float(len(nesPos))
        
        l1 = [ len(allPos), len(allHigherAndPos), len(nesPos), len(nesHigherAndPos)]

        allPos = allHigherAndPos = nesPos =  nesHigherAndPos = 1

        """

        #this could be speed up twice with the same accuracy! 
        if nes >= 0:
            allPos = int(len(vals) - numpy.searchsorted(nvals, 0, side="left"))
            allHigherAndPos = int(len(vals) - numpy.searchsorted(nvals, nes, side="left"))
            nesPos = len(nnes) - int(numpy.searchsorted(nnes, 0, side="left"))
            nesHigherAndPos = len(nnes) - int(numpy.searchsorted(nnes, nes, side="left"))
        else:
            allPos = int(numpy.searchsorted(nvals, 0, side="left"))
            allHigherAndPos = int(numpy.searchsorted(nvals, nes, side="right"))
            nesPos = int(numpy.searchsorted(nnes, 0, side="left"))
            nesHigherAndPos = int(numpy.searchsorted(nnes, nes, side="right"))
           
        """
        #Comparing results
        l2 = [ allPos, allHigherAndPos, nesPos, nesHigherAndPos ]
        diffs = [ l1[i]-l2[i] for i in range(len(l1)) ]
        sumd = sum( [ abs(a) for a in diffs ] )
        if sumd > 0:
            print nes > 0
            print "orig", l1
            print "modi", l2
        """

        try:
            top = allHigherAndPos/float(allPos) #p value
            down = nesHigherAndPos/float(nesPos)

            fdrs.append(top/down)
        except:
            fdrs.append(1000000000.0)
    
    #print "Whole part", time.time() - tb1

    return zip(enrichmentScores, nEnrichmentScores, enrichmentPVals, fdrs)

from . import obiGene

def nth(l,n): return [ a[n] for a in l ]

def itOrFirst(data):
    """ Returns input if input is of type ExampleTable, else returns first
    element of the input list """
    if iset(data):
        return data
    else:
        return data[0]

def wrap_in_list(data):
    """ Wraps orange.ExampleTable in a list """
    if iset(data):
        return [ data ]
    else:
        return data

def takeClasses(datai, classValues=None):
    """
    Function joins class groups specified in an input pair
    classValues. Each element of the pair is a list of class
    values to be joined to first or second class. Group
    classes in two new class values.
    If class values are not specified, take all the classes.

    Input data can be a single data set or a list of data sets
    with the same domain.

    Returns transformed data sets / data sets. 
    """

    cv = itOrFirst(datai).domain.classVar
    nclassvalues = None

    if cv and len(itOrFirst(datai)) > 1:
        oldcvals = [ a for a in cv.values ]
        
        if not classValues:
            classValues = oldcvals

        toJoin = []

        for vals in classValues:
            if issequencens(vals):
                toJoin.append(list(vals))
            else:
                toJoin.append([vals])

        classValues = reduce(lambda x,y: x+y, toJoin)
        classValues = [ str(a) for a in classValues ] # ok class values

        #dictionary of old class -> new class
        mapval = {}
        nclassvalues = [] # need to preserver order

        for joinvals in toJoin:
            joinvalsn = "+".join([ str(val) for val in sorted(joinvals) ])
            nclassvalues.append(joinvalsn)

            for val in joinvals:
                mapval[str(val)] = joinvalsn

        #take only examples with classValues classes
        nclass = orange.EnumVariable(cv.name, values=nclassvalues)
        ndom = orange.Domain(itOrFirst(datai).domain.attributes, nclass)

        def removeAndTransformClasses(data):
            """
            Removes unnecessary class values and joins them according
            to function input.
            """
            examples = []
            for ex in data:
                if ex[cv] in classValues:
                    nex = orange.Example(ndom, ex)
                    nex[-1] = mapval[str(ex[cv].value)]
                    examples.append(nex)

            return orange.ExampleTable(ndom, examples)

        if iset(datai):
            datai = removeAndTransformClasses(datai)
        else:
            datai = [ removeAndTransformClasses(data) for data in datai ]

    return datai

def removeBadAttributes(datai, atLeast=3):
    """
    Removes attributes which would obscure GSEA analysis.

    Attributes need to be continuous, they need to have
    at least one value. Remove other attributes.

    For the attribute to be valid, it needs to have at least
    [atLeast] values for every class value.

    Return transformed data set / data sets and ignored attributes.
    """

    def attrOk(a, data):
        """
        Attribute is ok if it is continouous and if containg
        at least atLest not unknown values.
        """

        a = data.domain.attributes.index(a)

        #can't
        if data.domain.attributes[a].varType != orange.VarTypes.Continuous:
            return False

        if len(data) == 1:

            vals = [ex[a].value for ex in data if not ex[a].isSpecial()]
            if len(vals) < 1:
                return False 
        
        if len(data) > 1 and data.domain.classVar and atLeast > 0:
            
            dc = dict( (v, 0) for v in data.domain.classVar.values )

            for ex in data:
                if not ex[a].isSpecial():
                    dc[ex[-1].value] += 1

            minl = min(dc.values())

            if minl < atLeast:
                #print "Less than atLeast"
                return False

        return True
    

    def notOkAttributes(data):
        ignored = []
        for a in data.domain.attributes:
            if not attrOk(a, data):
                #print "Removing", a
                ignored.append(a)
        return ignored
    
    ignored = []
    if iset(datai):
        ignored = set(notOkAttributes(datai))
    else:
        #ignore any attribute which is has less than atLeast values for each class
        #ignored = set(reduce(lambda x,y: x+y, [ notOkAttributes(data) for data in datai ]))

        #remove any attribute, which is ok in less than half of the dataset
        ignored = []
        for a in itOrFirst(datai).domain.attributes:
            attrOks = sum([ attrOk(a, data) for data in datai ])
            if attrOks < len(datai)/2:
                ignored.append(a)


    natts = [ a for a in itOrFirst(datai).domain.attributes if a not in ignored ]
    #print ignored, natts, set(ignored) & set(natts)

    ndom = orange.Domain(natts, itOrFirst(datai).domain.classVar)

    datao = None
    if iset(datai):
        datao = orange.ExampleTable(ndom, datai)
    else:
        datao = [ orange.ExampleTable(ndom, data) for data in datai ]

    return datao, ignored

def keepOnlyMeanAttrs(datai, atLeast=3, classValues=None):
    """
    Attributes need to be continuous, they need to have
    at least one value.

    In order of attribute to be valid, it needs to have at least
    [atLeast] values for every class value.

    Keep only specified classes - group them in two values.
    """    
    datai = takeClasses(datai, classValues=classValues)
    return removeBadAttributes(datai, atLeast=atLeast)

def data_single_meas_column(data):
    """ 
    Returns true if data seems to be in one column
    (float variables) only. This column should contain 
    the rankings
    """
    columns = [a for a in data.domain] +  [ data.domain.getmeta(a) for a in list(data.domain.getmetas()) ]
    floatvars = [ a for a in columns if a.varType == orange.VarTypes.Continuous ]
    if len(floatvars) == 1:
        return True
    else:
        return False

def transform_data(data, phenVar, geneVar):
    """
    if we have log2ratio in a single value column, transpose the matrix
    i.e. we have a single column with a continous variable. first
    string variable then becomes the gene name

    The goal is to have different phenotypes annotated with a class,
    and names of genes as attribute names.

    If phenVar is False, then we can work, then the input already
    consists of scores of differential expressions

    If we have a single column, transpose it. 
    If phenVar is one of the groups, transpose the matrix.
    """

    def prepare_data(data, phenVar=None, geneVar=None):

        def rorq(a, name):
            """ Group annatation or question mark. """
            try: 
                return a.attributes[name]
            except: 
                return '?'
   
        #use class as phenotype by default, if it is present,
        #if not, do not use any phenotype!
        if phenVar == None: 
            if not data.domain.classVar:
                phenVar = False
            else:
                phenVar = data.domain.classVar


        #TODO validate phenVar and geneVar?
        #TODO autodetection of groups?

        #transpose is not needed if phenVar is classVar or phenVar is False
        #and there is only one sample
        if phenVar == data.domain.classVar or \
            (phenVar == False and len(data) == 1):

            if geneVar == None: #if not specified, set as true in this stage
                geneVar = True

            floatvars = [ a for a in data.domain.attributes \
                if a.varType == orange.VarTypes.Continuous ]

            #rename attributes without touching the original variable
            if geneVar != True:
                fl2 = []

                for a in floatvars:
                    na = orange.FloatVariable(name=rorq(a, geneVar))
                    na.getValueFrom = lambda e, rw: e[a]
                    fl2.append(na)

                floatvars = fl2

            dom = orange.Domain(floatvars, phenVar)
            return orange.ExampleTable(dom, data)

        elif phenVar == False or phenVar != data.domain.classVar:

            cands = allgroups(data)
            pv = False
            if phenVar != False:
                pv = orange.EnumVariable(name="phenotype", 
                    values=list(cands[phenVar]))

            #take the only string attribute as a gene name
            gc = gene_cands(data, False)
            if geneVar == None:
                if len(gc) == 1:
                    geneVar = gc[0]
                else:
                    geneNamesUnspecifiedError()
           
            latts = [ orange.FloatVariable(name=ex[geneVar].value) \
                for ex in data ]

            domain = orange.Domain(latts, pv)

            examples = []
            for at in data.domain.attributes:
                if at.varType == orange.VarTypes.Continuous:
                    vals = [ ex[at].value for ex in data ]
                    if pv != False: #add class value
                        vals.append(rorq(at, phenVar))
                    examples.append(orange.Example(domain, vals))

            return orange.ExampleTable(domain, examples)
        else:
            wrongInputsError()

    #transform all example tables
    single = iset(data)
    transposed = [ prepare_data(d, phenVar, geneVar) for d in wrap_in_list(data) ]

    if single:
        return transposed[0]
    else:
        return transposed


def allgroups(data):
    """
    Return all phenotype descriptors of attributes with their values.
    """
    sd = defaultdict(set)
    for attr in data.domain.attributes:
        for key, value in attr.attributes.items():
            sd[key].add(value)
    return sd

def already_have_correlations(data):
    return len(data) == 1 or data_single_meas_column(data)

def need_to_transpose_single(data):
    if len(data) == 1:
        return False
    else:
        return True

def phenotype_cands(data):
    """
    Return all phenotype candidate descriptors in a list of tuples 
    (variable, values). Candidates are class variable, if it exists and 
    attributes dictionaries of attributes.
    Phenotype candidates must contain at least two differend values.
    """
    if already_have_correlations(data):
        return [ (False, set()) ]
    else:
        cv = []
        if data.domain.classVar and data.domain.classVar.varType == orange.VarTypes.Discrete:
            cv.append((data.domain.classVar, set(data.domain.classVar.values)))
        cands = cv + sorted(allgroups(data).items())
        return filter(lambda x: len(x[1]) >= 2, cands)

def gene_cands(data, correct):
    """
    Returns all valid gene descriptors with regards to the choosen
    phenotype variable.
    Return variable descriptor for variables, name of the group for
    descriptions in attr.attributes and True for the usage
    of attribute names.
    Correct is True, if the example table has genes as attributes.
    """
    if correct:
        #gene names could be in attributes or as gene names (marker True)
        return [True] + nth(sorted(allgroups(data)),0)
    else:
        #gene names are values of some string attribute
        columns = [a for a in data.domain] +  \
            [ data.domain.getmeta(a) for a in list(data.domain.getmetas()) ]
        stringvars = [ a for a in columns if a.varType == 6 ]
        return stringvars

def is_variable(phenVar):
    return isinstance(phenVar, orange.Variable)

class GSEA(object):

    def __init__(self, data, organism=None, matcher=None, classValues=None, 
        atLeast=3, caseSensitive=False, phenVar=None, geneVar=None):
        """
        If the data set constains multiple measurements for a single gene,
        all are considered. Individual constributions of such measurements
        are not weighted down - each measurement is as important as they
        would measure different genes.

        phenVar and geneVar can ether be an orange attribute or a string.
        If they are strings, then they describe a group.
        """

        self.genesets = {}
        self.organism = organism

        if organism != None:
            print "WARNING: obiGsea - organism and caseSensitive parameters are deprecated. Use matcher instead."

        self.gsweights = {}
        self.namesToIndices = None
        self.gm = matcher

        data = transform_data(data, phenVar, geneVar)

        data, info = keepOnlyMeanAttrs(data, classValues=classValues, atLeast=atLeast)

        self.data = data

        #init attrnames
        attrnames = [ a.name for a in itOrFirst(self.data).domain.attributes ]

        if self.gm == None: #build a gene matcher, if if does not exists
            self.gm = obiGene.matcher([obiGene.GMKEGG(self.organism, ignore_case=not caseSensitive)], 
                ignore_case=not caseSensitive, direct=True)
            print "WARNING: gene matcher build automatically for organism: " + self.organism

        self.gm.set_targets(attrnames)

 
    def addGeneset(self, genesetname, genes):
        """
        Add a single gene set. See addGenesets function.
        Solely for backwards compatibility.
        """
        self.addGenesets({ genesetname: genes })

    def addGenesets(self, genesets):
        """
        Adds genesets from input dictionary. Performs gene matching. Adds
        to a self.genesets: key is genesetname, it's values are individual
        genes and match results.
        """
        for g in obiGeneSets.GeneSets(genesets):
            genes = g.genes
            datamatch = filter(lambda x: x[1] != None, 
                [ (gene, self.gm.umatch(gene)) for gene in genes])
            self.genesets[g] = datamatch

    def selectGenesets(self, minSize=3, maxSize=1000, minPart=0.1):
        """ Returns a list of gene sets that have sizes in limits """

        def okSizes(orig, transl):
            """compares sizes of genesets to limitations"""
            if len(transl) >= minSize and len(transl) <= maxSize \
                and float(len(transl))/len(orig) >= minPart:
                return True
            return False

        return  dict( (a,c) for a,c in self.genesets.iteritems() if okSizes(a.genes,c) )

    def genesIndices(self, genes):
        """
        Returns in attribute indices of given genes.
        Buffers locations dictionary.
        """
        if not self.namesToIndices:
            self.namesToIndices = defaultdict(list)
            for i,at in enumerate(itOrFirst(self.data).domain.attributes):
                self.namesToIndices[at.name].append(i)
        return reduce(lambda x,y:x+y, [ self.namesToIndices[gname] for gname in genes ], [])

    def compute_gene_weights(self, gsweights, gsetsnum, nattributes):
        """
        Computes gene set weights for all specified weights.
        Expects gene sets in form { name: [ num_attributes ] }
        GSWeights are 
        """
        pass

    def to_gsetsnum(self, gsets):
        """
        Returns a dictionary of given  gene sets in gsetnums format.
        """
        return dict( (gs,self.genesIndices(nth(self.genesets[gs],1))) for gs in gsets)

    def compute(self, minSize=3, maxSize=1000, minPart=0.1, n=100, **kwargs):

        subsetsok = self.selectGenesets(minSize=minSize, maxSize=maxSize, minPart=minPart)

        geneweights = None

        gsetsnum = self.to_gsetsnum(subsetsok.keys())
        gsetsnumit = gsetsnum.items() #to fix order

        #gsetsnumit = gsetsnumit[:1]
        #print gsetsnumit

        if len(gsetsnum) == 0:
            return {} # quick return if no genesets

        if len(self.gsweights) > 0:
            #set geneset
            geneweights = [1]*len(data.domain.attributes)

        if len(itOrFirst(self.data)) > 1:
            gseal = gseaE(self.data, nth(gsetsnumit,1), n=n, geneweights=geneweights, **kwargs)
        else:
            rankings = [ self.data[0][at].native() for at in self.data.domain.attributes ]
            gseal = gseaR(rankings, nth(gsetsnumit,1), n=n, **kwargs)

        res = {}

        for gs, gseale in zip(nth(gsetsnumit,0),gseal):
            rdict = {}
            rdict['es'] = gseale[0]
            rdict['nes'] = gseale[1]
            rdict['p'] = gseale[2]
            rdict['fdr'] = gseale[3]
            rdict['size'] = len(gs.genes)
            rdict['matched_size'] = len(self.genesets[gs])
            rdict['genes'] = nth(self.genesets[gs],1)
            res[gs] = rdict

        return res

def runGSEA(data, organism=None, classValues=None, geneSets=None, n=100, 
        permutation="class", minSize=3, maxSize=1000, minPart=0.1, atLeast=3, 
        matcher=None, geneVar=None, phenVar=None, caseSensitive=False, 
        **kwargs):
    """
    phenVar and geneVar specify the phenotype and gene variable.
    """
    gso = GSEA(data, organism=organism, matcher=matcher, 
        classValues=classValues, atLeast=atLeast, caseSensitive=caseSensitive,
        geneVar=geneVar, phenVar=phenVar)
    gso.addGenesets(geneSets)
    res1 = gso.compute(n=n, permutation=permutation, minSize=minSize,
        maxSize=maxSize, minPart=minPart, **kwargs)
    return res1

def etForAttribute(datal,a):
    """
    Builds an example table for a single attribute across multiple 
    example tables.
    """

    tables = len(datal)

    def getAttrVals(data, attr):
        dom2 = orange.Domain([data.domain[attr]], False)
        dataa = orange.ExampleTable(dom2, data)
        return [ a[0].native() for a in dataa ]

    domainl = []
    valuesl = []

    for id, data in enumerate(datal):
        v = getAttrVals(data,a)
        valuesl.append(v)
        domainl.append(orange.FloatVariable(name=("v"+str(id))))

    classvals = getAttrVals(data, datal[0].domain.classVar)
    valuesl += [ classvals ]

    dom = orange.Domain(domainl, datal[0].domain.classVar)
    examples = [ list(a) for a in zip(*valuesl) ]

    datat = orange.ExampleTable(dom, examples)

    return datat


def evaluateEtWith(fn, *args, **kwargs):
    """
    fn - evaluates example table given
    following arguments.
    """

    def newf(datal):
        res = []
        for a in datal[0].domain.attributes:
            et = etForAttribute(datal, a)
            res.append(fn(et, *args, **kwargs))
        return res

    return newf


def hierarchyOutput(results, limitGenes=50):
    """
    Transforms results for use by hierarchy output from GO.

    limitGenes - maximum number of genes on output.
    """
    trans = []
    
    for name, res in results.items():
        try:
            second = name.split(' ')[2]
            name = second if second[:2] == 'GO' else name
        except:
            pass
        
        trans.append((name, abs(res["nes"]), res["matched_size"], res["size"], res["p"], min(res["fdr"], 1.0), res["genes"][:limitGenes]))

    return trans

if  __name__=="__main__":

    """
    Old example with another measure function
    data = orange.ExampleTable("gene_three_lines_log.tab")
    data = orange.ExampleTable("sterolTalkHepa.tab")
    gen1 = collections(['steroltalk.gmt', ':kegg:hsa'], default=False)
    rankingf = rankingFromOrangeMeas(MA_anova())
    matcher = obiGene.matcher([obiGene.GMKEGG('hsa')])
    geneVar = gene_cands(data, False)[1]
    phenVar = "group"
    out = runGSEA(data, n=10, geneSets=gen1, permutation="gene", atLeast=3, matcher=matcher, rankingf=rankingf, phenVar=phenVar, geneVar=geneVar)
    """
    """
    data = orange.ExampleTable("sterolTalkHepa.tab")
    gen1 = obiGeneSets.collections('steroltalk.gmt', (("KEGG",), "9606"))
    matcher = obiGene.matcher([obiGene.GMKEGG('hsa')])
    out = runGSEA(data, n=10, geneSets=gen1, permutation="gene", atLeast=3, matcher=matcher)
    """
    matcher = obiGene.matcher([[obiGene.GMKEGG('ddi'), obiGene.GMDicty()]])
    data = orange.ExampleTable("/home/marko/t_gsea1.tab")
    gen1 = obiGeneSets.collections((("KEGG",), "352472"))
    out = runGSEA(data, n=10, geneSets=gen1, permutation="gene", atLeast=3, matcher=matcher, phenVar="growth")
    print out
    print "\n".join(map(str,sorted(out.items())))
    



"""
<name>Data Files Selector</name>
<description>Selects a subset of data files.</description>
<icon>icons/DataFilesSelector.png</icon>
<priority>1060</priority>
<contact>Peter Juvan (peter.juvan@fri.uni-lj.si)</contact>
<prototype>1</prototype>
"""

from __future__ import absolute_import

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .OWDataFiles import DataFiles, ExampleSelection

class OWDataFilesSelector(OWWidget):
    settingsList  = ["applyOnChange"]

    def __init__(self, parent=None, signalManager = None):
        OWWidget.__init__(self, parent, signalManager, 'Data Files Selector', wantMainArea = 0, resizingEnabled = 1)

        self.callbackDeposit = []

        self.inputs = [("Structured Data", DataFiles, self.onDataInput)]
        self.outputs = [("Examples", ExampleTable), ("Structured Data", DataFiles)]

        self.dataStructure = None
        self.datasets = None
        self.lastSentIds = []

        # Settings
        self.applyOnChange = 0
        self.loadSettings()

        # GUI
        # info
        box = OWGUI.widgetBox(self.controlArea, "Info", addSpace = True)
        self.infoa = OWGUI.widgetLabel(box, 'No data loaded.')
        self.infob = OWGUI.widgetLabel(box, '')
        self.infoc = OWGUI.widgetLabel(box, '')
            
        # LIST VIEW
        frmListView = OWGUI.widgetBox(self.controlArea, None, addSpace = True)
        self.tree = QTreeWidget(frmListView)
        self.tree.setSelectionMode(QAbstractItemView.MultiSelection)
        self.tree.setHeaderLabel("Directory/Data File")
        frmListView.layout().addWidget(self.tree)
        self.connect(self.tree,SIGNAL('itemSelectionChanged()'),self.selectionChanged)

        # Output
        box = OWGUI.widgetBox(self.controlArea, "Output", addSpace = True)
        OWGUI.checkBox(box, self, 'applyOnChange', 'Commit data on selection change')
        self.commitBtn = OWGUI.button(box, self, "Commit", callback=self.sendData, disabled=1)
        self.resize(300,600)

    def setFileTree(self):
        self.disconnect(self.tree,SIGNAL('itemSelectionChanged()'),self.selectionChanged)
        self.tree.clear()
        self.listitems = []
        if self.dataStructure:
            for d in self.dataStructure:
                (dirname, files) = d
                diritem = QTreeWidgetItem(self.tree, [dirname], QTreeWidgetItem.UserType)
                diritem.setSelected(1)
                self.listitems.append(diritem)
                diritem.setExpanded(1)
                diritem.name = dirname
                for f in files:
                    item = QTreeWidgetItem(diritem, [f.name], QTreeWidgetItem.UserType)
                    item.setSelected(1)
                    self.listitems.append(item)
                    item.data = f
        self.connect(self.tree,SIGNAL('itemSelectionChanged()'),self.selectionChanged)

    def selectionChanged(self):
        if self.applyOnChange and self.okToCommit:
            self.sendData()

    # checks which data has been selected, builds a data structure, and sends it out
    def sendData(self):
        data = []
        # clear what has been previously sent
        for id in self.lastSentIds:
            self.send("Examples", None, id)
        self.lastSentIds = []
        # send new data
        id = 0
        for tIdx in range(self.tree.topLevelItemCount()):
            dir = self.tree.topLevelItem(tIdx)
            if dir in self.tree.selectedItems():
                files = []
                for cIdx in range(dir.childCount()):
                    f = dir.child(cIdx)
                    if f in self.tree.selectedItems():
                        files.append(f.data)
                        self.send("Examples", f.data, id)
                        self.lastSentIds.append(id)
                        id += 1
                data.append((dir.name, files))
        self.send("Structured Data", data)

    def onDataInput(self, dataStructure):
        self.dataStructure = dataStructure
        self.datasets = []
        if dataStructure and len(dataStructure):
            for name, etList in dataStructure:
                for et in etList:
                    self.datasets.append(et)
            # sumarize the data
            numSets = len(self.dataStructure)
            numFiles = len(self.datasets)
            self.infoa.setText("%d set%s, total of %d data file%s." % (numSets, ["", "s"][numSets!=1], numFiles, ["","s"][numFiles!=1]))
            # construct lists that sumarize the data
            numExamplesList = []
            numAttrList = []
            hasClass = []
            attrNameList = []
            for et in self.datasets:
                numExamplesList.append(len(et))
                numAttrList.append(len(et.domain.attributes))
                hasClass.append(et.domain.classVar != None)
                for attr in et.domain.attributes:
                    if attr.name not in attrNameList:
                        attrNameList.append(attr.name)
            # report the number of attributes/class
            if len(numAttrList):
                minNumAttr = min(numAttrList)
                maxNumAttr = max(numAttrList)
                if minNumAttr != maxNumAttr:
                    infob = "From %d to %d attribute%s (%d in total)" % (minNumAttr, maxNumAttr, ["","s"][maxNumAttr!=1], len(attrNameList))
                else:
                    infob = "%d attribute%s" % (maxNumAttr, ["","s"][maxNumAttr!=1])
            else:
                infob = "No attributes"
            if sum(hasClass) == len(hasClass):
                infob += ", all files contain class variable."
            elif sum(hasClass) == 0:
                infob += ", no class variable."
            else:
                infob += ", some files contain class variable."
            self.infob.setText(infob)
            # report the number of examples
            if len(numExamplesList):
                infoc = "Files contain "
                minNumE = min(numExamplesList)
                maxNumE = max(numExamplesList)
                if minNumE == maxNumE:
                    infoc += "%d example%s, " % (maxNumE, ["","s"][maxNumE!=1])
                else:
                    infoc += "from %d to %d example%s, " % (minNumE, maxNumE, ["","s"][maxNumE!=1])
                infoc += "%d in total." % sum(numExamplesList)
            else:
                infoc = "Files contain no examples."
            self.infoc.setText(infoc)

            # read data
            self.okToCommit = 1
            self.commitBtn.setEnabled(1)
        else:
            self.infoa.setText('No data on input.')
            self.infob.setText('')
            self.infoc.setText('')
            self.okToCommit = 0
            self.commitBtn.setEnabled(0)
        # enable/disable commit
        # read data
        self.setFileTree()
        if self.applyOnChange:
            self.sendData()


if __name__=="__main__":
    a=QApplication(sys.argv)
    ow=OWDataFilesSelector()
    ow.show()
    a.exec_()
    ow.saveSettings()



"""\
==============================================
KEGG - Kyoto Encyclopedia of Genes and Genomes
==============================================

This is a python module for access to `KEGG`_ using its web services. To use this module you need to have
`SUDS`_ python library installed (other backends are planed). 

.. _`KEGG`: http://www.genome.jp/kegg/

.. _`SUDS`: http://pypi.python.org/pypi/suds/

"""
from __future__ import absolute_import


import os, sys
from collections import defaultdict

from datetime import datetime

from Orange.utils import lru_cache, serverfiles

from . import databases
from . import entry

from .brite import BriteEntry, Brite

from . import api
from . import conf
from . import pathway

KEGGGenome = databases.Genome
KEGGGenes = databases.Genes
KEGGEnzymes = databases.Enzymes
KEGGReaction = databases.Reactions
KEGGPathways = databases.Pathways

KEGGBrite = Brite
KEGGBriteEntry = BriteEntry

KEGGPathway = pathway.Pathway

DEFAULT_CACHE_DIR = conf.params["cache.path"]


from .. import obiProb
from Orange.utils import deprecated_keywords, deprecated_attribute

class Organism(object):
    def __init__(self, org, genematcher=None):
        self.org_code = self.organism_name_search(org)
        self.genematcher = genematcher
        self.api = api.CachedKeggApi()
        
    @property
    def org(self):
        return self.org_code
    
    @property
    def genes(self):
        if not hasattr(self, "_genes"):
            genes = KEGGGenes(self.org_code)
            self._genes = genes
        return self._genes
    
    def gene_aliases(self):
        return self.genes().gene_aliases()
    
    def pathways(self, with_ids=None):
        if with_ids is not None:
            return self.api.get_pathways_by_genes(with_ids)
        else:
            return [p.entry_id for p in self.api.list_pathways(self.org_code)]
    
    def list_pathways(self):
        return self.pathways()
    
    def get_linked_pathways(self, pathway_id):
        self.api.get_linked_pathways(pathway_id)
        
    def enzymes(self, genes=None):
        raise NotImplementedError()
    
    def get_enriched_pathways(self, genes, reference=None, prob=obiProb.Binomial(), callback=None):
        """ Return a dictionary with enriched pathways ids as keys
        and (list_of_genes, p_value, num_of_reference_genes) tuples 
        as items.
        
        """
        allPathways = defaultdict(lambda :[[], 1.0, []])
        from Orange.orng import orngMisc
        milestones = orngMisc.progressBarMilestones(len(genes), 100)
        pathways_db = KEGGPathways()
        
        pathways_for_gene = []
        for i, gene in enumerate(genes):
            pathways_for_gene.append(self.pathways([gene]))
            if callback and i in milestones:
                callback(i*50.0/len(genes))
                
        # precache for speed 
        pathways_db.pre_cache([pid for pfg in pathways_for_gene for pid in pfg]) 
        for i, (gene, pathways) in enumerate(zip(genes, pathways_for_gene)):
            for pathway in pathways:
                if pathways_db.get_entry(pathway).gene: 
                    allPathways[pathway][0].append(gene)
            if callback and i in milestones:
                callback(50.0 + i*50.0/len(genes))
        reference = set(reference if reference is not None else self.genes.keys())
        
        pItems = allPathways.items()
        
        for i, (p_id, entry) in enumerate(pItems):
            pathway = pathways_db.get_entry(p_id)
            entry[2].extend(reference.intersection(pathway.gene or []))
            entry[1] = prob.p_value(len(entry[0]), len(reference), len(entry[2]), len(genes))
        return dict([(pid, (genes, p, len(ref))) for pid, (genes, p, ref) in allPathways.items()])
        
    def get_genes_by_enzyme(self, enzyme):
        enzyme = Enzymes().get_entry(enzyme)
        return enzyme.genes.get(self.org_code, []) if enzyme.genes else []
    
    def get_genes_by_pathway(self, pathway_id):
        return KEGGPathway(pathway_id).genes()
    
    def get_enzymes_by_pathway(self, pathway_id):
        return KEGGPathway(pathway_id).enzymes()
    
    def get_compounds_by_pathway(self, pathway_id):
        return KEGGPathway(pathway_id).compounds()
    
    def get_pathways_by_genes(self, gene_ids):
        return self.api.get_pathways_by_genes(gene_ids)
        gene_ids = set(gene_ids)
        pathways = [self.genes[id].pathway for id in gene_ids if self.genes[id].pathway]
        pathways = reduce(set.union, pathways, set())
        return [id for id in pathways if gene_ids.issubset(KEGGPathway(id).genes())] 
    
    def get_pathways_by_enzymes(self, enzyme_ids):
        enzyme_ids = set(enzyme_ids)
        pathways = [KEGGEnzymes()[id].pathway for id in enzyme_ids]
        pathwats = reduce(set.union, pathways, set())
        return [id for id in pathways if enzyme_ids.issubset(KEGGPathway(id).enzymes())]
    
    def get_pathways_by_compounds(self, compound_ids):
        compound_ids = set(compound_ids)
        pathways = [KEGGCompounds()[id].pathway for id in compound_ids]
        pathwats = reduce(set.union, pathways, set())
        return [id for id in pathways if compound_ids.issubset(KEGGPathway(id).compounds())]
    
    def get_enzymes_by_compound(self, compound_id):
        return KEGGCompound()[compound_id].enzyme
    
    def get_enzymes_by_gene(self, gene_id):
        return self.genes[gene_id].enzymes
    
    def get_compounds_by_enzyme(self, enzyme_id):
        return self._enzymes_to_compounds.get(enzyme_id)
    
    @deprecated_keywords({"caseSensitive": "case_sensitive"})
    def get_unique_gene_ids(self, genes, case_sensitive=True):
        """Return a tuple with three elements. The first is a dictionary mapping from unique gene
        ids to gene names in genes, the second is a list of conflicting gene names and the third is a list
        of unknown genes.
        """
        unique, conflicting, unknown = {}, [], []
        for gene in genes:
            names = self.genematcher.match(gene)
            if len(names) == 1:
                unique[names[0]] = gene
            elif len(names) == 0:
                unknown.append(gene)
            else:
                conflicting.append(gene)
        return unique, conflicting, unknown
    
    def get_genes(self):
        return self.genes
    
    @classmethod
    def organism_name_search(cls, name):
        genome = KEGGGenome()
        if name not in genome:
            ids = genome.search(name)
            if not ids:
                from .. import obiTaxonomy
                ids = obiTaxonomy.search(name)
                ids = [id for id in ids if genome.search(id)]
            name = ids.pop(0) if ids else name
            
        try:
            return genome[name].entry_key
        except KeyError:
            raise ValueError("Organism with name='%s' not found in KEGG." % name)
        
    @classmethod
    def organism_version(cls, name):
        name = cls.organism_name_search(name)
        genome = KEGGGenome()
        info = genome.api.binfo(name)
        return info.release
#        orngServerFiles.localpath_download("KEGG", "kegg_genes_%s.tar.gz" % name)
#        return orngServerFiles.info("KEGG", "kegg_genes_%s.tar.gz" % name)["datetime"]
    
    def _set_genematcher(self, genematcher):
        setattr(self, "_genematcher", genematcher)
        
    def _get_genematcher(self):
        if getattr(self, "_genematcher", None) == None:
            from .. import obiGene
            if self.org_code == "ddi":
                self._genematcher = obiGene.matcher([obiGene.GMKEGG(self.org_code), obiGene.GMDicty(),
                                                     [obiGene.GMKEGG(self.org_code), obiGene.GMDicty()]])
            else:
                self._genematcher = obiGene.matcher([obiGene.GMKEGG(self.org_code)])
            self._genematcher.set_targets(self.genes.keys())
        return self._genematcher
    
    genematcher = property(_get_genematcher, _set_genematcher)
    
KEGGOrganism = Organism
    
def organism_name_search(name):
    return KEGGOrganism.organism_name_search(name)

def pathways(org):
    return KEGGPathway.list(org)

def organisms():
    return KEGGOrganism.organisms()

def from_taxid(taxid):
    genome = KEGGGenome()
    res = genome.search(taxid)
    print taxid, res
    for r in res:
        e = genome[r]
        
        if e.taxid in [taxid,  genome.TAXID_MAP.get(taxid, taxid)]:
            return e.org_code()
        
    return None

def to_taxid(name):
    genome = KEGGGenome()
    if name in genome:
        return genome[name].taxid
    
    keys = genome.search(name)
    if keys:
        return genome[keys[0]].taxid
    else:
        return None

def create_gene_sets():
    pass

from .. import obiGene
from Orange.utils import ConsoleProgressBar

class MatcherAliasesKEGG(obiGene.MatcherAliasesPickled):
    DOMAIN = "KEGG"
    VERSION = "v3.0"
    def create_aliases(self):
        import cPickle
        files = set(serverfiles.ServerFiles().listfiles(self.DOMAIN))
        ids_filename = "kegg_gene_id_aliases_" + self.organism + ".pickle"
        if ids_filename in files:
            filename = serverfiles.localpath_download(self.DOMAIN, ids_filename)
            
            aliases = cPickle.load(open(filename, "rb"))
        else:
            pb = ConsoleProgressBar("Retriving KEGG ids:")
            kegg_org = KEGGOrganism(self.organism)
            genes = kegg_org.genes
            genes.pre_cache(progress_callback=pb.set_state)
            aliases = []
            for key, entry in genes.iteritems():
                aliases.append(set([key]) | set(entry.alt_names))
            filename = serverfiles.localpath_download(self.DOMAIN, ids_filename)
            cPickle.dump(aliases, open(filename, "wb"))
            
        return aliases
    
    def filename(self):
        return "kegg3_" + self.organism
    
    def aliases_path(self):
        ids_filename = "kegg_gene_id_aliases_" + self.organism + ".pickle"
        return serverfiles.localpath(self.DOMAIN, ids_filename)
    
    def create_aliases_version(self):
        files = set(serverfiles.listfiles(self.DOMAIN))
        ids_filename = "kegg_gene_id_aliases_" + self.organism + ".pickle"
        if ids_filename in files:
            version = serverfiles.info(self.DOMAIN, ids_filename)["datetime"]
        else:
            kegg_org = KEGGOrganism(self.organism)
            genes = kegg_org.genes
            version = genes.info.release
        return version
        
    def __init__(self, organism, **kwargs):
        self.organism = organism
        sf = serverfiles.ServerFiles()
        files = set(sf.listfiles(self.DOMAIN))
        ids_filename = "kegg_gene_id_aliases_" + self.organism + ".pickle"
        if ids_filename in files:
            serverfiles.update(self.DOMAIN, ids_filename)
            
        obiGene.MatcherAliasesPickled.__init__(self, **kwargs)

def main():
    KEGGGenome()
    import doctest
    extraglobs = {"api": KeggApi()}
    doctest.testmod(optionflags=doctest.ELLIPSIS, extraglobs=extraglobs)

if __name__ == "__main__":
    sys.exit(main())
